@@ -47,8 +47,6 @@
 #include <openssl/bio.h>
 #endif
 
-#include "afflib_sha256.h"
-
 #include "vnode_raw.h"
 #include "vnode_split_raw.h"
 #include "vnode_afm.h"
@@ -85,7 +83,11 @@ struct af_vnode *af_vnode_array[] = {
     &vnode_ewf,				// libewf
 #endif
 #ifdef USE_QEMU
-    &vnode_qemu,
+    &vnode_vmdk,
+    &vnode_dmg,
+#endif
+#ifdef USE_SPARSEIMAGE			
+    &vnode_sparseimage,           
 #endif
     &vnode_split_raw,			// must be before raw
     &vnode_raw,				// greedy; must be last
@@ -97,18 +99,9 @@ struct af_vnode *af_vnode_array[] = {
  *** Support functions that don't use the "af"
  ****************************************************************/
 
-#if defined(HAVE_SHA256_INIT) 
-int af_SHA256(const unsigned char *d,size_t s,unsigned char *md);
-#endif
-
-
 static int aff_initialized = 0;
 int af_cache_debug = 0;
 FILE *af_trace = 0;
-static unsigned char buf4k_sha256[] = 
-    {0x90,0x18,0xbb,0xab,0x39,0x27,0x49,0xb2,0x83,0x1f,0xba,
-     0x33,0xc3,0xf7,0xdf,0xed,0x67,0xae,0x11,0x8d,0x88,0x3f,
-     0xe9,0x2a,0xe4,0x21,0xf7,0xfa,0x7d,0xa7,0x24,0x8c};
 
 void af_initialize()
 {
@@ -118,27 +111,10 @@ void af_initialize()
     assert(sizeof(struct af_head)==8);
     assert(sizeof(struct af_segment_head)==16);
     assert(sizeof(struct af_segment_tail)==8);
-
     assert(sizeof(struct affkey)==AFFKEY_SIZE);
 
-#ifdef HAVE_SHA256_INIT
-    /* Validate SHA256 */
-    unsigned char buf4k[4096];
-    unsigned char md256[SHA256_DIGEST_LENGTH];
-    memset(buf4k,0,sizeof(buf4k));
-    buf4k[1] = 1;
-    SHA256_CTX cc;
-    SHA256_Init(&cc);
-    SHA256_Update(&cc,buf4k,sizeof(buf4k));
-    SHA256_Final(md256,&cc);
-    for(int i=0;i<SHA256_DIGEST_LENGTH;i++){
-	assert(md256[i]==buf4k_sha256[i]);
-    }
-    af_SHA256(buf4k,sizeof(buf4k),md256);
-    for(int i=0;i<SHA256_DIGEST_LENGTH;i++){
-	assert(md256[i]==buf4k_sha256[i]);
-    }
-#endif
+    /* Make sure OpenSSL is working */
+    OpenSSL_add_all_algorithms();
 
     const char *val = getenv(AFFLIB_CACHE_DEBUG);
     if(val) af_cache_debug = atoi(val);
@@ -357,7 +333,12 @@ AFFILE *af_open_with(const char *url,int flags,int mode, struct af_vnode *v)
     const char *cache_pages = getenv(AFFLIB_CACHE_PAGES);
     if(cache_pages) af->num_pbufs = atoi(cache_pages);
     if(af->num_pbufs<1) af->num_pbufs = AFFLIB_CACHE_PAGES_DEFAULT; // default valuen
+
     af->pbcache   = (struct aff_pagebuf *)calloc(af->num_pbufs,sizeof(struct aff_pagebuf));
+    if(af->pbcache==0){			// if can't allocate the full amount
+	af->num_pbufs = 2;		// try a significantly smaller cache
+	af->pbcache   = (struct aff_pagebuf *)calloc(af->num_pbufs,sizeof(struct aff_pagebuf));
+    }
 
     if(flags & AF_HALF_OPEN) return af;	// for low-level tools
 
@@ -369,12 +350,13 @@ AFFILE *af_open_with(const char *url,int flags,int mode, struct af_vnode *v)
     }
 
     /* If there is no AFFKEY and the file is read-only, don't use a password */
-    if(af->password && (af_get_seg(af,AF_AFFKEY,0,0,0)!=0) && ((af->openflags&O_ACCMODE)==O_RDONLY)){
+    if(af->password && (af_get_seg(af,AF_AFFKEY,0,0,0)!=0) && ((af->openflags & O_ACCMODE)==O_RDONLY)){
 	af_sanitize_password(af);
     }
 
     /* Set up the encryption if requested and if this support metadata */
     if(AF_SEALING_VNODE(af)){
+	bool can_decrypt = false;
 	if(af->password){
 	    struct af_vnode_info vni;
 	    memset(&vni,0,sizeof(vni));
@@ -385,20 +367,22 @@ AFFILE *af_open_with(const char *url,int flags,int mode, struct af_vnode *v)
 		}
 		if(r==0){
 		    r = af_use_aes_passphrase(af,af->password);
-		    if(r) (*af->error_reporter)("af_open: invalid passphrase: '%s'",af->password);
+		    if(r==0) {
+			can_decrypt = true;
+		    } else {
+			(*af->error_reporter)("af_open: invalid passphrase: '%s'",af->password);
+		    }
 		}
 		af_sanitize_password(af);
-		if(r!=0){
-		    af_deallocate(af);
-		    return 0;
-		}
 	    }
 	}
 	
 	/* Try public key... */
-	const char *kf = getenv(AFFLIB_DECRYPTING_PRIVATE_KEYFILE);
-	if(kf){
-	    af_set_unseal_keyfile(af,kf);
+	if(can_decrypt==false){
+	    const char *kf = getenv(AFFLIB_DECRYPTING_PRIVATE_KEYFILE);
+	    if(kf){
+		af_set_unseal_keyfile(af,kf);
+	    }
 	}
     }
 	
@@ -432,11 +416,19 @@ AFFILE *af_open(const char *filename,int flags,int mode)
     return 0;				// can't figure it out; must be an invalid extension
 }
 
+/** Set an option and return the previous value */
 int af_set_option(AFFILE *af,int option,int value)
 {
+    int prev = 0;
     switch(option){
-    case AF_OPTION_AUTO_ENCRYPT: af->crypto->auto_encrypt = value;return 0;
-    case AF_OPTION_AUTO_DECRYPT: af->crypto->auto_decrypt =  value;return 0;
+    case AF_OPTION_AUTO_ENCRYPT:
+	prev = af->crypto->auto_encrypt;
+	af->crypto->auto_encrypt = value;
+	return prev;
+    case AF_OPTION_AUTO_DECRYPT:
+	prev = af->crypto->auto_decrypt;
+	af->crypto->auto_decrypt = value;
+	return prev;
     }
     return -1;
 }
@@ -499,19 +491,20 @@ uint64_t af_seek(AFFILE *af,int64_t pos,int whence)
 {
     AF_WRLOCK(af);
     if(af_trace) fprintf(af_trace,"af_seek(%p,%"I64d",%d)\n",af,pos,whence);
-    int64_t new_pos=0;
+    uint64_t new_pos=0;
     switch(whence){
     case SEEK_SET:
 	new_pos = pos;
 	break;
     case SEEK_CUR:
-	new_pos = af->pos + pos;
+	if(pos<0 && ((uint64_t)(-pos)) > af->pos) new_pos=0;
+	else new_pos = af->pos + pos;
 	break;
     case SEEK_END:
-	new_pos = af->image_size - pos;
+	if((uint64_t)pos > af->image_size) new_pos=0;
+	else new_pos = af->image_size - pos;
 	break;
     }
-    if(new_pos < 0) new_pos = 0;	// can't be less than 0
 
     /* Note if the direction has changed */
     int direction = (new_pos > af->pos)  ? 1 : ((new_pos < af->pos) ? -1 : 0);
@@ -613,6 +606,7 @@ int64_t af_get_imagesize(AFFILE *af)
 {
     int64_t ret = -1;
     struct af_vnode_info vni;
+    memset(&vni,0,sizeof(vni));
     if(af_vstat(af,&vni)==0){
 	/* If vni.imagesize is 0 and if there are encrypted segments and if there
 	 * is no imagesize segment but there is an encrypted one, then we can't read this encrypted file...
@@ -821,6 +815,7 @@ int af_get_next_seg(AFFILE *af,char *segname,size_t segname_len,unsigned long *a
 
 int af_rewind_seg(AFFILE *af)
 {
+    if(af_trace) fprintf(af_trace,"af_rewind_seg(%p)\n",af);
     AF_READLOCK(af);
     if(af->v->rewind_seg==0){
 	errno = ENOTSUP;
@@ -838,6 +833,7 @@ int af_rewind_seg(AFFILE *af)
 int af_update_segf(AFFILE *af, const char *segname,
 		  unsigned long arg,const u_char *data,unsigned int datalen,u_int flag)
 {
+    if(af_trace) fprintf(af_trace,"af_update_segf(%p,segname=%s,arg=%lu,datalen=%d)\n",af,segname,arg,datalen);
     AF_WRLOCK(af);
     if(af->v->update_seg==0){
 	errno = ENOTSUP;
@@ -845,21 +841,16 @@ int af_update_segf(AFFILE *af, const char *segname,
 	return -1;	// not supported by this file system
     }
 
-    if(af->vni_cache){
-	free(af->vni_cache);
-	af->vni_cache = 0;
-    }
+    af_invalidate_vni_cache(af);
 
     /* See if we need to encrypt. New memory might need to be allocated.
      * This isn't a big deal, because encryption requires copying memory
      * in any event; it's either an in-place copy or a copy to another location.
      */
-    const u_char *signdata = data;	// remember the original data location
 #ifdef HAVE_AES_ENCRYPT
     const char *oldname = 0;
     unsigned char *newdata = 0;
-    if(AF_SEALING(af)
-       && ((flag & AF_SIGFLAG_NOSEAL)==0)){
+    if(AF_SEALING(af) && ((flag & AF_SIGFLAG_NOSEAL)==0) && af->crypto->auto_encrypt){
 	/* Create an IV */
 	unsigned char iv[AES_BLOCK_SIZE];
 	memset(iv,0,sizeof(iv));
@@ -886,7 +877,7 @@ int af_update_segf(AFFILE *af, const char *segname,
 	datalen += pad + extra;
     }
 #endif
-    int r = (*af->v->update_seg)(af,segname,arg,data,datalen);
+    int r = (*af->v->update_seg)(af,segname,arg,data,datalen); // actually update the segment
     if(r==0) af->bytes_written += datalen;
 #ifdef HAVE_AES_ENCRYPT
     /* if we encrypted, make sure the unencrypted segment is deleted */
@@ -896,12 +887,22 @@ int af_update_segf(AFFILE *af, const char *segname,
 	newdata = 0;
     }
 #endif
+    /* If we wrote out an unencrypted segment, make sure that the corresopnding encrypted
+     * segment is deleted.
+     */
+    char encrypted_name[AF_MAX_NAME_LEN];
+    strlcpy(encrypted_name,segname,sizeof(encrypted_name));
+    strlcat(encrypted_name,AF_AES256_SUFFIX,sizeof(encrypted_name));
+    if(*af->v->del_seg) (*af->v->del_seg)(af,encrypted_name); // no need to check error return
+
+
     /* Sign the segment if:
      * - there is a signing private key
      * - the data structure and flag not set
      * - This is not a signature segment
      */
 #ifdef USE_AFFSIGS
+    const u_char *signdata = data;	// remember the original data location
     if(AF_SEALING(af)
        && (r==0)
        && af->crypto->sign_privkey
@@ -915,7 +916,7 @@ int af_update_segf(AFFILE *af, const char *segname,
     return r;
 }
 
-/* Requires no locking */
+/* Requires no locking because locking is done in af_update_segf */
 int af_update_seg(AFFILE *af, const char *segname,
 		  unsigned long arg,const u_char *data,unsigned int datalen)
 {
@@ -956,6 +957,14 @@ int af_del_seg(AFFILE *af,const char *segname)
     return ret;
 }
 
+void af_invalidate_vni_cache(AFFILE *af)
+{
+    if(af->vni_cache){
+	free(af->vni_cache);
+	af->vni_cache = 0;
+    }
+}
+
 int af_vstat(AFFILE *af,struct af_vnode_info *vni)
 {
     AF_READLOCK(af);
@@ -998,3 +1007,9 @@ void af_stats(AFFILE *af,FILE *f)
 }
 
 
+int af_set_acquisition_date(AFFILE *af,time_t t)
+{
+    char timebuf[64];
+    strftime(timebuf,sizeof(timebuf),"%Y-%m-%d %H:%M:%S\n",localtime(&t));
+    return af_update_seg(af,AF_ACQUISITION_DATE,0,(const u_char *)timebuf,strlen(timebuf));
+}
