@@ -31,38 +31,46 @@
 #include <inttypes.h>
 #endif
 
-/* WIN32 is defined by the NMAKE makefile for Visual C++ under Windows */
+/** WIN32 is defined by the NMAKE makefile for Visual C++ under Windows and by mingw **/
 #ifdef WIN32                            
 #include <io.h>				// gets isatty
 typedef unsigned int uint;
 typedef unsigned int u_int;
 typedef unsigned long ulong;
 typedef unsigned long u_long;
-typedef unsigned _int64 uint64;		/* 64-bit types Types */
-typedef          _int64 int64;
+typedef unsigned __int64 uint64;		/* 64-bit types Types */
+typedef          __int64 int64;
 typedef unsigned char u_char;
+
 #ifndef _UINT64_T_DECLARED
-typedef unsigned _int64 uint64_t;	/* 64-bit types Types */
+typedef unsigned __int64 uint64_t;	/* 64-bit types Types */
 #define _UINT64_T_DECLARED
+#endif
+
+#ifndef _INT64_T_DECLARED
+typedef         __int64 int64_t;
+#define _INT64_T_DECLARED
+#endif
 
 #ifndef PRId64
 #define PRId64 "I64d"
 #endif
+
 #ifndef PRIi64
 #define PRIi64 "I64i"
 #endif
+
 #ifndef PRIu64
 #define PRIu64 "I64u"
 #endif
-#endif
 
-#ifndef _INT64_T_DECLARED
-typedef          _int64 int64_t;
-#define _INT64_T_DECLARED
-#endif
-int64   ftello(FILE *stream);	 /* Functions that Microsoft forgot */
-int     fseeko(FILE *stream,int64 offset,int whence);
+//int64   ftello(FILE *stream);	 /* Functions that Microsoft forgot */
+//int     fseeko(FILE *stream,int64 offset,int whence);
+#define ftello ftello64
+#define fseeko fseeko64
 #endif
+/** END OF WIN32 DEFINES **/
+
 
 #define I64d PRIi64
 #define I64u PRIu64
@@ -97,6 +105,7 @@ struct af_vnode_info {
     u_int page_count_total;
     u_int segment_count_signed;
     u_int segment_count_encrypted;
+    u_int page_count_encrypted;
 };					// 
 
 
@@ -144,17 +153,18 @@ void	af_stats(AFFILE *af,FILE *f);	// print stats to f
 void	af_set_cachesize(AFFILE *af,int max); // how much memory can the cache use?
 int     af_vstat(AFFILE *af,struct af_vnode_info *vni); // does the stat
 void	af_perror(const char *fname);				// print the error string to stderr
-void	af_err(int code,const char *fname,...);				// like err(), but the aff version
+void	af_err(int code,const char *fname,...);	// like err(), but will also print AFF-specific errors
 
 
 
 /* Generic set/get option routines; this replaces individual options in previous implementations.
- * af==0 to set global options.
+ * af==0 to set global options. Return the previous value.
  */
 int	af_set_option(AFFILE *af,int option,int value);
 
 #define AF_OPTION_AUTO_ENCRYPT     1	// 1 = auto-encrypt
 #define AF_OPTION_AUTO_DECRYPT     2	// 1 = auto-decrypt
+// The following are not implemented yet 
 #define AF_OPTION_PIECEWISE_MD5    3	// 1 = automatically write pagen_md5 segments
 #define AF_OPTION_PIECEWISE_SHA1   4	// 1 = automatically write pagen_md5 segments
 #define AF_OPTION_PIECEWISE_SHA256 5	// 1 = automatically write pagen_md5 segments
@@ -166,10 +176,8 @@ int	af_set_option(AFFILE *af,int option,int value);
 #define AF_BADBLOCK_FILL  (1<<30)	// fill unallocated (sparse) with BADBLOCK flag
 #define AF_HALF_OPEN      (1<<29)       // return af before calling af->v->open; 
 
-
-
 /* navigating within the data segments as if they were a single file */
-int	af_read(AFFILE *af,unsigned char *buf,size_t count);
+ssize_t	af_read(AFFILE *af,unsigned char *buf,ssize_t count);
 uint64_t  af_seek(AFFILE *af,int64_t pos,int whence); // returns new position
 uint64_t  af_tell(AFFILE *af);
 int	af_eof(AFFILE *af);		// is the virtual file at the end?
@@ -192,6 +200,8 @@ int	    af_identify(AFFILE *af);	// returns type of AFFILE pointer
 /* Accessor Functions */
 int64_t	    af_get_imagesize(AFFILE *af);	// byte # of last mapped byte in image, or size of device;
 					// returns -1 if error
+int	    af_set_acquisition_date(AFFILE *af,time_t t); // sets AF_ACQUISITION_DATE
+
 #define af_imagesize(af) af_get_imagesize(af) // backwards compatiability 
 int	    af_get_segq(AFFILE *af,const char *name,int64_t *quad);/* Get/set 8-byte values */
 int	    af_update_segq(AFFILE *af,const char *name,int64_t quad);
@@ -246,12 +256,16 @@ int	af_del_seg(AFFILE *af,const char *name); // complete delete of first name
  *     to be the function used.
  */
 int64_t	af_segname_page_number(const char *name); // return -1 if it is not a page number
-int64_t	af_segname_hash_page_number(const char *name,char *hash,int hashlen);
+int64_t	af_segname_hash_page_number(const char *name,char *hash,int hashlen); // return -1 if it is not a hash page #
+
+int af_display_as_quad(const char *segname); // afflib recommends displaying this segment as an 8-byte quad
+int af_display_as_hex(const char *segname); // afflib recommends displaying this segment as a hex-string
 
 /****************************************************************/
 
 /* Crypto */
 /* AFF Base Encryption */
+int  af_SHA256(const unsigned char *buf,size_t buflen,unsigned char md[32]); // return 0 if success, -1 if no cipher
 int  af_set_aes_key(AFFILE *af,const unsigned char *userKey,const int bits);
 int  af_cannot_decrypt(AFFILE *af);	// encrypted pages are present which cannot be decrypted
 int  af_has_encrypted_segments(AFFILE *af);
@@ -326,6 +340,8 @@ int af_hash_verify_seg2(AFFILE *af,const char *segname,u_char *sigbuf_,size_t si
 #define AF_PAGE_SHA1	AF_PAGE"_sha1"	// sha1 hash of page
 #define AF_PAGE_SHA256	AF_PAGE"_sha256"// sha256 hash of page
 #define AF_PARITY0      "parity0"	// parity page of all bytes
+#define AF_BATCH_NAME		"batch_name"
+#define AF_BATCH_ITEM_NAME	"batch_item_name"
 
 #define AF_BLANKSECTORS "blanksectors"	// all NULs; 8-bytes
 #define AF_AFF_FILE_TYPE "aff_file_type" // contents should be "AFF", "AFM" or "AFD"
@@ -369,8 +385,8 @@ int af_hash_verify_seg2(AFFILE *af,const char *segname,u_char *sigbuf_,size_t si
 #define AF_PAGE_COMP_ALG_LZMA   0x0020	// high compression but pretty slow
 #define AF_PAGE_COMP_ALG_ZERO   0x0030  // Data segment is a 4-byte value of # of NULLs. 
 
-#define AF_MD5  "md5"			// stores image md5
-#define AF_SHA1 "sha1"			// stores image sha1
+#define AF_MD5    "md5"			// stores image md5
+#define AF_SHA1   "sha1"			// stores image sha1
 #define AF_SHA256 "sha256"		// stores image sha256
 
 #define AF_CREATOR	"creator"	// progname of the program that created the AFF file
@@ -449,7 +465,7 @@ int af_hash_verify_seg2(AFFILE *af,const char *segname,u_char *sigbuf_,size_t si
 #define AFFLIB_CACHE_STATS      "AFFLIB_CACHE_STATS" // make non-zero to dump stats to STDERR at end
 #define AFFLIB_CACHE_DEBUG      "AFFLIB_CACHE_DEBUG" // make "1" to dump a trace of cache events to stderr
 #define AFFLIB_CACHE_PAGES      "AFFLIB_CACHE_PAGES" // Size of the page cache
-#define AFFLIB_CACHE_PAGES_DEFAULT 2	// default number of cache pages
+#define AFFLIB_CACHE_PAGES_DEFAULT 32	// default number of cache pages
 #define AFFLIB_BIGTMP           "AFFLIB_BIGTMP" // default directory to put very big files for test programs
 #define AFFLIB_TRACEFILE        "AFFLIB_TRACEFILE" // If set, send a record of all activity to the location
 /* passphrases for single-key cryptography */
