@@ -6,10 +6,12 @@
 #include "affconfig.h"
 #include "afflib.h"
 #include "afflib_i.h"
-#include "afflib_sha256.h"		// make sure we have a SHA256 implementation
 #include "base64.h"
 #include "aftimer.h"
+
+#ifdef HAVE_THREADED_HASH_H
 #include "threaded_hash.h"
+#endif
 
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
@@ -20,7 +22,7 @@ extern "C" {
 #include "LzmaRamDecode.h"
 }
 
-#define MAX_FMTS 10000			// how many formats we should write
+int MAX_FMTS = 10000;		// how many formats we should write
 
 const char *fmt = "%8d Another format string.\n"; // must be constant size
 const char *progname = 0;
@@ -29,6 +31,7 @@ const char *opt_protocol = "file:///";
 const char *opt_ext = "aff";
 int   opt_compression_level = AF_COMPRESSION_DEFAULT;// default compression level
 int   opt_compression_type  = AF_COMPRESSION_ALG_ZLIB;	// 
+const char *tempdir = "/tmp/";
 
 
 /* Create the segment that we need */
@@ -40,30 +43,39 @@ int   opt_compression_type  = AF_COMPRESSION_ALG_ZLIB;	//
 
 const char *filename(char *buf,int buflen,const char *base)
 {
-    snprintf(buf,buflen,"%s%s.%s",opt_protocol,base,opt_ext);
+    snprintf(buf,buflen,"%s%s%s.%s",opt_protocol,tempdir,base,opt_ext);
     return buf;
 }
 
-void enable_writing(AFFILE *af)
+AFFILE *open_testfile(const char *base,int wipe)
 {
-    af_enable_compression(af,opt_compression_type,opt_compression_level);
-    af_set_pagesize(af,1024);
-    af_set_maxsize(af,(int64_t)65536);		// force splitting of raw and afd files
+    int flags = O_CREAT|O_RDWR;
+    char fn[1024];
+
+    filename(fn,sizeof(fn),base);
+    printf("%s = %s\n",base,fn);
+    if(wipe){
+	unlink(fn);				// make sure it is gone
+	flags |= O_TRUNC;
+    }
+    AFFILE *af = af_open(fn,flags,0666);
+    if(!af) err(1,"af_open");
+    if(wipe){
+	af_enable_compression(af,opt_compression_type,opt_compression_level);
+	af_set_pagesize(af,1024);
+	af_set_maxsize(af,(int64_t)65536);		// force splitting of raw and afd files
+    }
+    return af;
 }
 
 int sequential_test()
 {
     char buf[1024];
+    const char *fmt = "this is line %d\n";
 
     printf("Sequential test...\n");
-
-    char fn[1024];
-
-    filename(fn,sizeof(fn),"test_sequential");
-    unlink(fn);				// make sure it is gone
-    AFFILE *af = af_open(fn,O_CREAT|O_RDWR|O_TRUNC,0666);
-    if(!af) err(1,"af_open");
-    enable_writing(af);
+    
+    AFFILE *af = open_testfile("test_sequential",1);
     for(int i=0;i<MAX_FMTS;i++){
 	if(i%250==0) printf("\rwriting %d/%d...",i,MAX_FMTS);
 	sprintf(buf,fmt,i);
@@ -81,7 +93,7 @@ int sequential_test()
     printf("\nSequential file written.\n");
     printf("\n");
     printf("Now verifying the string...\n");
-    af = af_open(fn,O_RDONLY,0666);
+    af = open_testfile("test_sequential",0);
     if(!af) err(1,"af_open");
     for(int i=0;i<MAX_FMTS;i++){
 	char rbuf[1024];
@@ -92,7 +104,7 @@ int sequential_test()
 	}
 	rbuf[len] = 0;			// terminate the string
 	if(strcmp(buf,rbuf)!=0){
-	    err(1,"Attempt to verify entry %d failed.\nExpected: (len=%d) '%s'\nGot: (len=%d) '%s'\n",
+	    err(1,"Attempt to verify entry %d failed.\nExpected: (len=%zd) '%s'\nGot: (len=%zd) '%s'\n",
 		i,strlen(buf),buf,strlen(rbuf),rbuf);
 	}
     }
@@ -106,19 +118,11 @@ int reverse_test()
 {
     char wbuf[1024];
     char rbuf[1024];
-    char fn[1024];
-    int pass = 0;
 
     printf("Reverse write test...\n");
-    filename(fn,sizeof(fn),"test_reverse");
-    unlink(fn);
-
     for(int pass=1;pass<=2;pass++){
-	int mode = (pass==1) ? O_CREAT|O_RDWR|O_TRUNC : O_RDONLY;
 
-	AFFILE *af = af_open(fn,mode,0666);
-	if(!af) err(1,"af_open");
-	if(pass==1) enable_writing(af);
+	AFFILE *af = open_testfile("test_reverse",pass==1);
 	for(int i=MAX_FMTS-1;i>=0;i--){
 	    sprintf(wbuf,fmt,i);
 	    af_seek(af,strlen(wbuf)*i,SEEK_SET);
@@ -148,9 +152,8 @@ int reverse_test()
 
 int random_write_test()
 {
-    char fn[1024];
     char buf[1024];
-    char tally[MAX_FMTS];
+    char *tally = (char *)calloc(MAX_FMTS,1);
     int i;
 
     memset(tally,0,sizeof(tally));
@@ -162,12 +165,7 @@ int random_write_test()
     printf("Random write test...\n");
     printf("Creating test file with  %d byte records.\n", fmt_size);
 
-    filename(fn,sizeof(fn),"test_random");
-    unlink(fn);				// make sure it is gone
-    AFFILE *af = af_open(fn,O_CREAT|O_RDWR|O_TRUNC,0666);
-    if(!af) err(1,"af_open");
-    enable_writing(af);
-
+    AFFILE *af = open_testfile("test_random",1);
 
     if(af_write(af,(unsigned char *)buf,fmt_size)!=fmt_size){
 	err(1,"af_write");
@@ -195,8 +193,7 @@ int random_write_test()
     
     /* Now verify what was written */
     printf("Verifying write test...\n");
-    af = af_open(fn,O_RDONLY,0);
-    if(!af) err(1,"af_open");
+    af = open_testfile("test_random",0);
     
     for(i=0;i<MAX_FMTS;i++){
 	char should[256];		// what we should get
@@ -230,11 +227,7 @@ int random_read_test(int total_bytes,int data_page_size)
 		     O_CREAT|O_RDWR|O_TRUNC|O_BINARY,0666);
     if(fd<0) err(1,"fopen");
 
-    char fn[1024];
-    AFFILE *af = af_open(filename(fn,sizeof(fn),"test_random_contents"),
-			 O_CREAT|O_RDWR|O_TRUNC,0666);
-    if(!af) err(1,"af_open");
-    enable_writing(af);
+    AFFILE *af = open_testfile("test_random_contents",1);
 
     /* Just write it out as one big write */
 
@@ -373,7 +366,7 @@ void sparse_test()
     af_set_maxsize(af,(int64_t)1024*1024*256);
     af_set_pagesize(af,1024*1024*16);
 
-    for(uint i=0;i<10;i++){
+    for(u_int i=0;i<10;i++){
 	uint64_t pos = mult*i;
 	memset(buf,0,sizeof(buf));
 	snprintf(buf,sizeof(buf),"This is at location=%"I64u"\n",pos);
@@ -382,7 +375,7 @@ void sparse_test()
     }
 
     /* Now verify */
-    for(uint i=0;i<10;i++){
+    for(u_int i=0;i<10;i++){
 	uint64_t pos = mult*i;
 	uint64_t q;
 	af_seek(af,pos,SEEK_SET);
@@ -405,11 +398,11 @@ void sparse_test()
     /* Now seek to somewhere that no data has been written and see if we get 0s. */
     memset(buf,'g',sizeof(buf));
     af_seek(af,mult/2,SEEK_SET);
-    int r = af_read(af,(unsigned char *)buf,sizeof(buf));
+    ssize_t r = af_read(af,(unsigned char *)buf,sizeof(buf));
     if(r!=sizeof(buf)){
-	err(1,"Tried to read %d bytes at mult/2; got %d bytes\n",sizeof(buf),r);
+	err(1,"Tried to read %zd bytes at mult/2; got %zd bytes\n",sizeof(buf),r);
     }
-    for(int i=0;i<sizeof(buf);i++){
+    for(u_int i=0;i<sizeof(buf);i++){
 	if(buf[i]!=0) err(1,"data error; buf[%d]=%d\n",i,buf[i]);
     }
 
@@ -418,7 +411,7 @@ void sparse_test()
     af_seek(af,9*mult,SEEK_SET);
     r = af_read(af,big_buf,sizeof(big_buf));
     if(r!=sizeof(buf)){
-	errx(1,"Tried to read %d bytes at the end of the file; got %d bytes (should get %d)",
+	errx(1,"Tried to read %zd bytes at the end of the file; got %zd bytes (should get %zd)",
 	    sizeof(big_buf),r,sizeof(buf));
     }
 			  
@@ -426,7 +419,7 @@ void sparse_test()
     /* Now see if we can read past the end of the file */
     af_seek(af,11*mult,SEEK_SET);
     r = af_read(af,(unsigned char *)buf,sizeof(buf));
-    if(r!=0) errx(1,"Tried to read past end of file; got %d bytes (should get 0)",r);
+    if(r!=0) errx(1,"Tried to read past end of file; got %zd bytes (should get 0)",r);
 
     af_close(af);
     printf("\nSprase test passes.\n");
@@ -443,8 +436,8 @@ void figure(const char *fn)
 	err(1,"af_figure_media");
     }
     printf("sector size: %d\n",afb.sector_size);
-    printf("total sectors: %qd\n",afb.total_sectors);
-    printf("max read blocks: %d\n",afb.max_read_blocks);
+    printf("total sectors: %"PRId64"\n",afb.total_sectors);
+    printf("max read blocks: %"PRId64"\n",afb.max_read_blocks);
     exit(0);
 }
 
@@ -469,7 +462,7 @@ void compress(const char *fname)
 
 void lzma_test()
 {
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) && !defined(__APPLE__)
     _malloc_options = "XARV";
 #endif
     //char *fn = "/usr/share/dict/web2";
@@ -539,7 +532,7 @@ int aestest()
     u_char test[1024],buf[1024],rbuf[1024];
     size_t  buflen = sizeof(buf);
     make_test_seg(test,0);
-    for(int len=0;len<=strlen((const char *)test);len++){
+    for(u_int len=0;len<=strlen((const char *)test);len++){
 	if(af_update_seg(af,"page0",0,test,len)) err(1,"af_update_seg len=%d",len);
 	
 	/* Now try to read the segment */
@@ -549,7 +542,7 @@ int aestest()
 	    err(1,"Could not read encrypted segment with length %d.\n",len);
 	}
 	if(buflen!=len){
-	    printf("size of returned segment = %d ",buflen);
+	    printf("size of returned segment = %zd ",buflen);
 	    printf("(should be %d) \n",len);
 	    exit(0);
 	}
@@ -593,7 +586,7 @@ int aestest()
     /* Try to read a segment that doesn't eixst */
     buflen = 0;
     if(af_get_seg(af,"encrypted2",0,0,&buflen)==0){
-	errx(1,"Error: Attempt to get size of non-existant segment 'encrypted2' got %d\n",buflen);
+	errx(1,"Error: Attempt to get size of non-existant segment 'encrypted2' got %zd\n",buflen);
     }
     af_close(af);
 
@@ -687,11 +680,12 @@ void readfile_test(const char *fname)
 	err(1,"af_open(%s)",fname);
     }
     printf("using '%s'\n",af->v->name);
-    printf("af_get_imagesize()=%qd errno=%d\n",af_get_imagesize(af),errno);
+    printf("af_get_imagesize()=%"PRId64" errno=%d\n",af_get_imagesize(af),errno);
 
     int r = af_read(af,buf,sizeof(buf));
     printf("af_read(af,buf,1024)=%d  errno=%d\n",r,errno);
-    fwrite(buf,1,512,stdout);
+    r = fwrite(buf,1,512,stdout);
+    assert(r==512);
     af_close(af);
     exit(0);
 }
@@ -700,7 +694,7 @@ void zap(const char *fn)
 {
     unsigned char buf[1024*1024];
     AFFILE *af = af_open(fn,O_RDWR,0666);
-    if(!af) err(1,"af_open(%s)");
+    if(!af) err(1,"af_open(%s)",fn);
     memset(buf,0,sizeof(buf));
     if(af_write(af,buf,sizeof(buf))!=sizeof(buf)){
 	err(1,"af_write()");
@@ -724,39 +718,9 @@ void bugs_test()
 void rsatest();
 void xmltest(const char *fn);
 
-void usage()
-{
-    printf("usage: %s [options]\n",progname);
-    printf("    -e ext = use ext for extension (default is %s)\n",opt_ext);
-    printf("    -p protocol = use protocol for protocol (default is %s)\n",opt_protocol);
-    printf("    -a = do all tests (except -L)\n");
-    printf("    -b = do the bugs test (tests things that were reported and fixed)\n");
-    printf("    -1 = do sequential test\n");
-    printf("    -2 = do reverse test\n");
-    printf("    -3 = do random write test\n");
-    printf("    -4 = do random read test\n");
-    printf("    -5 = do maxsize multi-file test\n");
-    printf("    -6 = sparse file test\n");
-    printf("    -B = run large file test (needs 5GB of disk)\n");
-    printf("    -L = use LZMA compression\n");
-    printf("    -r# = Repeat the random tests # times.\n");
-    printf("    -d<dir> = use <dir> as the working dir for files (default is /tmp)\n");
-    printf("    -f<dev> = run af_figure_media on dev and print the results\n");
-    printf("    -c filename = compress filename and output to stdout\n");
-    printf("    -T = just test the LZMA compression\n");
-    printf("    -q = quite; just report errors\n");
-    printf("    -R filename = just try to read the first 1024 bytes of a file and print what happens...\n");
-    printf("    -t = run some timing tests\n");
-    printf("    -v = verbose\n");
-#ifdef HAVE_AES_ENCRYPT
-    printf("    -C = just test AES encryption\n");
-#endif
-    printf("    -x fn = xml test\n");
-    printf("    -z fn = open up fn for writing and zap it.\n");
-}
-
 void time_test()
 {
+#ifdef HAVE_THREADED_HASH_H
     int size = 1024*1024*256;
     unsigned char *buf1 = (u_char *)calloc(size,1);
     int threaded=0;
@@ -766,13 +730,11 @@ void time_test()
 	buf1[0]= count;
 	for(threaded=0;threaded<2;threaded++){
 	    aftimer t;
-	    threaded_hash h_md5(EVP_md5,threaded);
-	    threaded_hash h_sha1(EVP_sha1,threaded);
-	    threaded_hash h_sha2(EVP_sha1,threaded);
-	    threaded_hash h_sha3(EVP_sha1,threaded);
-#ifdef HAVE_EVP_SHA256
-	    threaded_hash h_sha256(EVP_sha256,threaded);
-#endif
+	    threaded_hash h_md5(EVP_get_digestbyname("md5"),threaded);
+	    threaded_hash h_sha1(EVP_get_digestbyname("sha1"),threaded);
+	    threaded_hash h_sha2(EVP_get_digestbyname("sha1"),threaded);
+	    threaded_hash h_sha3(EVP_get_digestbyname("sha1"),threaded);
+	    threaded_hash h_sha256(EVP_get_digestbyname("sha256"),threaded);
 	    
 	    printf("Threaded: %d size: %d\n",threaded,size);
 	    t.start();
@@ -780,9 +742,7 @@ void time_test()
 	    h_sha2.update(buf1,size);
 	    h_sha3.update(buf1,size);
 	    h_md5.update(buf1,size);
-#ifdef HAVE_EVP_SHA256
 	    h_sha256.update(buf1,size);
-#endif
 	    
 	    unsigned char md[32];
 	    char b[64];
@@ -791,188 +751,29 @@ void time_test()
 	    h_sha1.final(md,sizeof(md)); printf("sha1: %s\n",af_hexbuf(b,sizeof(b),md,20,0));
 	    h_sha2.final(md,sizeof(md)); printf("sha2: %s\n",af_hexbuf(b,sizeof(b),md,20,0));
 	    h_sha3.final(md,sizeof(md)); printf("sha3: %s\n",af_hexbuf(b,sizeof(b),md,20,0));
-#ifdef HAVE_EVP_SHA256
 	    h_sha256.final(md,sizeof(md)); printf("sha256: %s\n",af_hexbuf(b,sizeof(b),md,32,0));
-#endif
 	    t.stop();
 	    printf("time: %g\n",t.elapsed_seconds());
 	    printf("==============================\n");
 	}
     }
+#endif
     exit(0);
 }
 
 
-int main(int argc,char **argv)
-{
-    progname = argv[0];
-    int do_bugs = 0;
-    int do_sequential = 0;
-    int do_reverse    = 0;
-    int do_random_write_test = 0;
-    int do_random_read_test  = 0;
-    int do_large_file = 0;
-    int do_maxsize_test = 0;
-    int random_repeat = 1;
-    int do_sparse_test = 0;
-    int do_all=0;
-    int ch;
-    const char *dir = getenv(AFFLIB_BIGTMP);	// use by default
-
-    setvbuf(stdout,0,_IONBF,0);
-    //putenv(AFFLIB_CACHE_STATS"=1");
-
-    if(argc==1){
-	printf("running all tests with -a option (exception bigfile test)\n");
-	do_all = 1;
-    }
-
-    while ((ch = getopt(argc, argv, "b123456aBLd:h?f:e:c:TCp:rx:R:z:t")) != -1) {
-	switch(ch){
-	case 'R':
-	    readfile_test(optarg);
-	    break;
-	case 'b':
-	    do_bugs = 1;
-	    break;
-	case '1':
-	    do_sequential = 1;
-	    break;
-	case '2':
-	    do_reverse = 1;
-	    break;
-	case '3':
-	    do_random_write_test = 1;
-	    break;
-	case '4':
-	    do_random_read_test = 1;
-	    break;
-	case '5':
-	    do_maxsize_test = 1;
-	    break;
-	case '6':
-	    do_sparse_test = 1;
-	    break;
-	case 'l':
-	    random_repeat = atoi(optarg);
-	    break;
-	case 'B':
-	    do_large_file = 1;
-	    break;
-	case 't': time_test(); break;
-	case 'L': opt_compression_type = AF_COMPRESSION_ALG_LZMA; break;
-	case 'T': lzma_test(); break;
-	case 'r': rsatest(); break;
-	case 'a':
-	    do_all = 1;
-	    break;
-	case 'z':
-	    zap(optarg);break;
-	case 'd': dir = optarg; break;
-	case 'f': figure(optarg); break;
-	case 'e': opt_ext = optarg; break;
-	case 'c': compress(optarg); break;
-	    
-	case 'p': opt_protocol = optarg; break;
-	case 'x': xmltest(optarg);break;
-	case 'C': aestest(); break;
-	case 'h':
-	case '?':
-	default:
-	    usage();
-	}
-    }
-	
-    if(dir){
-	fprintf(stderr,"Changing to %s\n",dir);
-	if(chdir(dir)) err(1,"Can't change directory to %s",optarg);
-    }
-
-    if(do_bugs || do_all) bugs_test();
-    if(do_sequential || do_all) sequential_test();
-    if(do_reverse || do_all ) reverse_test();
-    if(do_maxsize_test || do_all) maxsize_test();
-    if(do_sparse_test || do_all) sparse_test();
-
-    for(int i=0;i<random_repeat;i++){
-	if(do_random_read_test  || do_all) random_read_test(256*1024,rand() % 65536); 
-	if(do_random_write_test || do_all) random_write_test();
-    }
-
-    if(do_large_file) large_file_test();
-
-    /* Now erase the files ... */
-    unlink("test_random.aff");
-    unlink("test_reverse.aff");
-    unlink("test_random_contents.aff");
-    unlink("test_sequential.aff");
-    unlink("bugs.aff");
-    unlink("test_random_contents.img");
-    unlink("sparse.aff");
-    return 0;
-}
-
-
 #include <openssl/pem.h>
 #include <openssl/bio.h>
  
 void rsatest()
 {
-#if 0
-    int fd = open("public.pem",O_RDONLY,0);
-    struct stat sb;
-    fstat(fd,&sb);
-    int pub_key_size = sb.st_size;
-    char *pub_key = (char *)malloc(pub_key_size);
-    read(fd,pub_key,pub_key_size);
-    
-    fd = open("slg.signed",O_RDONLY,0);
-    fstat(fd,&sb);
-    int signed_data_size = sb.st_size;
-    unsigned char *signed_data = (unsigned char *)malloc(signed_data_size);
-    read(fd,signed_data,signed_data_size);
+    const EVP_MD *sha256 = EVP_get_digestbyname("sha256");
 
-    //    if(BIO* bio = BIO_new_mem_buf(pub_key, pub_key_size)){
-    BIO *sbio = BIO_new_file("private.pem","r");
-    if(!sbio) err(1,"private.pem");
-    RSA *skey = 0;
-    if(PEM_read_bio_RSAPrivateKey(sbio,&skey,NULL,NULL)==0){
-	err(1,"could not convert private key");
+    if(!sha256){
+	fprintf(stderr,"SHA256 not available\n");
+	return;
     }
 
-
-    BIO *pbio = BIO_new_file("public.pem", "r");
-    if(!pbio) err(1,"public.pem");
-    RSA *pkey = 0;
-    if(PEM_read_bio_RSA_PUBKEY(pbio,&pkey,NULL,NULL)==0){
-	err(1,"could not read public key");
-    }
-
-    /* Try to encrypt with public key and decrypt with private one */
-    char ptext[128];
-    memset(ptext,0,sizeof(ptext));
-    strcpy(ptext,"Simson Garfinkel");
-    char etext[128];
-    memset(etext,0,sizeof(etext));
-    if(RSA_public_encrypt(32,(unsigned char *)ptext,
-			  (unsigned char *)etext,pkey,RSA_PKCS1_OAEP_PADDING)==-1){
-	err(1,"rsa encrypt failed");
-    }
-
-    //BIO *plaintext = BIO_new_mem_buf(mydata,strlen(mydata+1));
-
-    /* Now try to decrypt */
-    char dtext[128];
-    memset(dtext,0,sizeof(dtext));
-    if(RSA_private_decrypt(128,(unsigned char *)etext,
-			   (unsigned char *)dtext,skey,RSA_PKCS1_OAEP_PADDING)==-1){
-	err(1,"rsa decrypt failed");
-    }
-    printf("decrypt done. dtext=%s\n",dtext);
-    RSA_free(skey);
-#endif
-
-#ifdef HAVE_EVP_sha256
     printf("Now try signing with X.509 certificates and EVP\n");
 
     char ptext[16];
@@ -987,7 +788,7 @@ void rsatest()
     EVP_MD_CTX md;
     EVP_PKEY *pkey = PEM_read_bio_PrivateKey(bp,0,0,0);
 
-    EVP_SignInit(&md,EVP_sha256());
+    EVP_SignInit(&md,sha256);
     EVP_SignUpdate(&md,ptext,sizeof(ptext));
     EVP_SignFinal(&md,sig,&siglen,pkey);
 
@@ -997,71 +798,27 @@ void rsatest()
     PEM_read_bio_X509(bp,&x,0,0);
     EVP_PKEY *pubkey = X509_get_pubkey(x);
     
-    printf("pubkey=%x\n",pubkey);
+    printf("pubkey=%p\n",pubkey);
 	
-    EVP_VerifyInit(&md,EVP_sha256());
+    EVP_VerifyInit(&md,sha256);
     EVP_VerifyUpdate(&md,ptext,sizeof(ptext));
     int r = EVP_VerifyFinal(&md,sig,siglen,pubkey);
     printf("r=%d\n",r);
 
     printf("do it again...\n");
-    EVP_VerifyInit(&md,EVP_sha256());
+    EVP_VerifyInit(&md,sha256);
     EVP_VerifyUpdate(&md,ptext,sizeof(ptext));
     r = EVP_VerifyFinal(&md,sig,siglen,pubkey);
     printf("r=%d\n",r);
 
     printf("make a tiny change...\n");
     ptext[0]='f';
-    EVP_VerifyInit(&md,EVP_sha256());
+    EVP_VerifyInit(&md,sha256);
     EVP_VerifyUpdate(&md,ptext,sizeof(ptext));
     r = EVP_VerifyFinal(&md,sig,siglen,pubkey);
     printf("r=%d\n",r);
-#endif
-    
-#if 0
-    X509 *pkey = 0;
-    if(PEM_read_bio_X509(bp,&pkey,NULL,NULL)==0) err(1,"couldn't read signing key");
-    
-    EVP_sha256()
-    RSA *skey = 0;
-    if(PEM_read_bio_RSAPrivateKey(bp,&skey,NULL,NULL)==0){
-	err(1,"could not read signing_key.pem");
-    }
-    BIO_free(bp);
-    printf("the message is short:");
-
-    int i = RSA_sign(NID_sha256,(const u_char *)ptext,sizeof(ptext),sig,&siglen,skey);
-    printf("i=%d siglen=%d\n",i,siglen);
-    printf("verify the signature now...\n");
-    RSA *pk = 0;
-    X509 *x=0;
-
-    if(PEM_read_bio_X509(bp,&x,NULL,NULL)==0) err(1,"couldn't read public key");
-    //pk = X509_get_X509_PUBKEY(x);
-    //printf("PEM_read_bio_RSA_PUBKEY=%d\n",j);
-    //int k = RSA_verify(NID_sha256,(const u_char *)ptext,sizeof(ptext),sig,siglen,pk);
-    int k = X509_verify(NID_sha256,(const u_char *)ptext,sizeof(ptext),sig,siglen,pk);
-    printf("RSA_verify=%d\n",k);
-    printf("make a change...\n");
-    ptext[0]='f';
-    k = RSA_verify(NID_sha256,(const u_char *)ptext,sizeof(ptext),sig,siglen,pk);
-    printf("RSA_verify=%d\n",k);
-#endif
 }
 
-#if 0
-RSA* rsa_key = 0;
-if(PEM_read_bio_RSA_PUBKEY(bio, &rsa_key, NULL, NULL)) {
-    printf("signed_data_size=%d\n",signed_data_size);
-    printf("RSA_size(rsa_key)=%d\n",RSA_size(rsa_key));
-    uint8_t* dst = new uint8_t[signed_data_size];
-    RSA_public_decrypt(signed_data_size, signed_data,
-		       dst, rsa_key, RSA_PKCS1_PADDING);
-
-    BIO_free(bio);
-}
-#endif
-
 void xmlseg(BIO *bp,AFFILE *af,const char *segname)
 {
     BIO_printf(bp,"  <segment>\n");
@@ -1085,12 +842,168 @@ void xmltest(const char *fn)
 {
     BIO *bp = BIO_new(BIO_s_mem());
     AFFILE *af = af_open(fn,O_RDONLY,0);
-    if(!af) err(1,fn);
+    if(!af) err(1,"%s",fn);
     char segname[AF_MAX_NAME_LEN];
     while(af_get_next_seg(af,segname,sizeof(segname),0,0,0)==0){
 	xmlseg(bp,af,segname);
     }
     char *buf=0;
-    size_t len = BIO_get_mem_data(bp,&buf);
-    fwrite(buf,1,len,stdout);
+    ssize_t len = BIO_get_mem_data(bp,&buf);
+    int r = fwrite(buf,1,len,stdout);
+    assert(r==len);
 }
+
+
+void image_test()
+{
+    char fn[1024];
+    printf("Imaging test...\n");
+    filename(fn,sizeof(fn),"test_image");
+    unlink(fn);				// make sure it is gone
+    AFFILE *af = af_open(fn,O_CREAT|O_RDWR|O_TRUNC,0666);
+    if(!af) err(1,"af_open");
+}
+
+void usage()
+{
+    printf("usage: %s [options]\n",progname);
+    printf("    -e ext = use ext for extension (default is %s)\n",opt_ext);
+    printf("    -p protocol = use protocol for protocol (default is %s)\n",opt_protocol);
+    printf("    -a = do all tests (except -L)\n");
+    printf("    -b = do the bugs test (tests things that were reported and fixed)\n");
+    printf("    -1 = do sequential test\n");
+    printf("    -2 = do reverse test\n");
+    printf("    -3 = do random write test\n");
+    printf("    -4 = do random read test\n");
+    printf("    -5 = do maxsize multi-file test\n");
+    printf("    -6 = sparse file test\n");
+    printf("    -B = run large file test (needs 5GB of disk)\n");
+    printf("    -L = use LZMA compression\n");
+    printf("    -r = perform RSA tests\n");
+    printf("    -d<dir> = use <dir> as the working dir for files (default is %s)\n",tempdir);
+    printf("    -D<filename> = write debugging trace to <filename>\n");
+    printf("    -f<dev> = run af_figure_media on dev and print the results\n");
+    printf("    -c filename = compress filename and output to stdout\n");
+    printf("    -T = just test the LZMA compression\n");
+    printf("    -q = quite; just report errors\n");
+    printf("    -R filename = just try to read the first 1024 bytes of a file and print what happens...\n");
+    printf("    -t = run some timing tests\n");
+    printf("    -n nn = sets MAX_FMTS (default %d)\n",MAX_FMTS);
+    printf("    -i image write speed test (lots of small pages)\n");
+    printf("    -v = verbose\n");
+#ifdef HAVE_AES_ENCRYPT
+    printf("    -C = just test AES encryption\n");
+#endif
+    printf("    -x fn = xml test\n");
+    printf("    -z fn = open up fn for writing and zap it.\n");
+}
+
+int main(int argc,char **argv)
+{
+    progname = argv[0];
+    int do_bugs = 0;
+    int do_sequential = 0;
+    int do_reverse    = 0;
+    int do_random_write_test = 0;
+    int do_random_read_test  = 0;
+    int do_large_file = 0;
+    int do_maxsize_test = 0;
+    int random_repeat = 1;
+    int do_sparse_test = 0;
+    int do_all=0;
+    int do_image_test =1;
+    int ch;
+
+    const char *bigdir = getenv(AFFLIB_BIGTMP);	// use by default
+
+    if(bigdir) tempdir = bigdir;
+
+    setvbuf(stdout,0,_IONBF,0);
+    //putenv(AFFLIB_CACHE_STATS"=1");
+
+    if(argc==1){
+	printf("running all tests with -a option (exception bigfile test)\n");
+	do_all = 1;
+    }
+
+    while ((ch = getopt(argc, argv, "b123456aBLd:h?f:e:c:TCp:rx:R:z:tn:D:")) != -1) {
+	switch(ch){
+	case 'D': af_trace = fopen(optarg,"w");break;
+	case 'R': readfile_test(optarg); break;
+	case 'b':
+	    do_bugs = 1;
+	    break;
+	case '1':
+	    do_sequential = 1;
+	    break;
+	case '2':
+	    do_reverse = 1;
+	    break;
+	case '3':
+	    do_random_write_test = 1;
+	    break;
+	case '4':
+	    do_random_read_test = 1;
+	    break;
+	case '5':
+	    do_maxsize_test = 1;
+	    break;
+	case '6': do_sparse_test = 1; break;
+	case 'l':
+	    random_repeat = atoi(optarg);
+	    break;
+	case 'B':
+	    do_large_file = 1;
+	    break;
+	case 'n': MAX_FMTS = atoi(optarg); break;
+	case 't': time_test(); break;
+	case 'L': opt_compression_type = AF_COMPRESSION_ALG_LZMA; break;
+	case 'T': lzma_test(); break;
+	case 'r': rsatest(); break;
+	case 'a':
+	    do_all = 1;
+	    break;
+	case 'z':
+	    zap(optarg);break;
+	case 'd': tempdir = optarg; break;
+	case 'f': figure(optarg); break;
+	case 'e': opt_ext = optarg; break;
+	case 'c': compress(optarg); break;
+	    
+	case 'p': opt_protocol = optarg; break;
+	case 'x': xmltest(optarg);break;
+	case 'C': aestest(); break;
+	case 'i': do_image_test=1;break;
+	case 'h':
+	case '?':
+	default:
+	    usage();
+	}
+    }
+	
+    if(do_bugs || do_all) bugs_test();
+    if(do_sequential || do_all) sequential_test();
+    if(do_reverse || do_all ) reverse_test();
+    if(do_maxsize_test || do_all) maxsize_test();
+    if(do_sparse_test || do_all) sparse_test();
+    if(do_image_test || do_all) image_test();
+
+    for(int i=0;i<random_repeat;i++){
+	if(do_random_read_test  || do_all) random_read_test(256*1024,rand() % 65536); 
+	if(do_random_write_test || do_all) random_write_test();
+    }
+
+    if(do_large_file) large_file_test();
+
+    /* Now erase the files ... */
+    unlink("test_random.aff");
+    unlink("test_reverse.aff");
+    unlink("test_random_contents.aff");
+    unlink("test_sequential.aff");
+    unlink("bugs.aff");
+    unlink("test_random_contents.img");
+    unlink("sparse.aff");
+    return 0;
+}
+
+
