@@ -19,7 +19,7 @@
 #endif
 
 
-#ifdef WIN32
+#if defined(WIN32) and !defined(__MINGW_H)
 /**********************************************************************
  * Implement dirent-style opendir/readdir/rewinddir/closedir on Win32
  *
@@ -178,19 +178,6 @@ static inline struct afd_private *AFD_PRIVATE(AFFILE *af)
 }
 
 
-static bool last4_is_afd(const char *filename)
-{
-
-    return af_ext_is(filename,"afd");
-}
-
-
-static bool last4_is_aff(const char *filename)
-{
-    return af_ext_is(filename,"aff");
-}
-
-
 /* afd_file_with_seg:
  * Returns the AFFILE for a given segment, or 0 if it isn't found.
  */
@@ -229,12 +216,12 @@ static int afd_identify_file(const char *filename,int exists)
     struct stat sb;
     if(stat(fn,&sb)==0){
 	if((sb.st_mode & S_IFMT)==S_IFDIR){
-	    if(last4_is_afd(fn)) return 1;
+	    if(af_ext_is(fn,"afd")) return 1;
 	}
 	return 0;			//
     }
     /* Doesn't exist. Does it end .afd ? */
-    if(last4_is_afd(fn)) return 1;
+    if(af_ext_is(fn,"afd")) return 1;
     return 0;
 }
 
@@ -374,7 +361,7 @@ static int afd_open(AFFILE *af)
     }
     struct dirent *dp;
     while ((dp = readdir(dirp)) != NULL){
-	if (last4_is_aff(dp->d_name)){
+	if (af_ext_is(dp->d_name,"aff")){
 	    char path[MAXPATHLEN+1];
 	    strlcpy(path,af->fname,sizeof(path));
 	    strlcat(path,"/",sizeof(path));
