@@ -19,36 +19,39 @@
 
 
 
-static int	aff_ignore_overhead();	
-static int      aff_write_ignore(AFFILE *af,unsigned long bytes);
-static int	aff_write_seg(AFFILE *af,const char *name,unsigned long arg, const u_char *value,unsigned int vallen); 
-static int	aff_get_seg(AFFILE *af,const char *name,unsigned long *arg,unsigned char *data,size_t *datalen);
+static int      aff_write_ignore(AFFILE *af,size_t bytes);
+static int	aff_write_seg(AFFILE *af,const char *name,unsigned long arg,
+			      const u_char *value,size_t vallen); 
+static int	aff_get_seg(AFFILE *af,const char *name,unsigned long *arg,
+			    unsigned char *data,size_t *datalen);
 static int	aff_get_next_seg(AFFILE *af,char *segname,size_t segname_len,
 				 unsigned long *arg, unsigned char *data, size_t *datalen);
 
-/* af_write_ignore:
- * write an AF_IGNORE structure of a given size.
+/** aff_segment_overhead:
+ * @param segname - the name of a segment
+ * @return The number of bytes in the AFF file that the segment takes up without the data.
  */
 
-static int aff_ignore_overhead()
+int aff_segment_overhead(const char *segname)
 {
-    return sizeof(struct af_segment_head)+sizeof(struct af_segment_tail);
+    return sizeof(struct af_segment_head)+sizeof(struct af_segment_tail)+(segname?strlen(segname):0);
 }
 
-static int aff_write_ignore2(AFFILE *af,unsigned long bytes)
+static int aff_write_ignore2(AFFILE *af,size_t bytes)
 {
+    if(af_trace) fprintf(af_trace,"aff_write_ignore2(%p,%zd)\n",af,bytes);
     unsigned char *invalidate_data = (unsigned char *)calloc(bytes,1);
     aff_write_seg(af,AF_IGNORE,0,invalidate_data,bytes); // overwrite with NULLs
     free(invalidate_data);
     return 0;
 }
 
-static int aff_write_ignore(AFFILE *af,unsigned long bytes)
+static int aff_write_ignore(AFFILE *af,size_t bytes)
 {
     int64_t startpos = ftello(af->aseg);	// remember start position
     int r = 0;
 
-    //printf("***delete %d bytes\n",bytes);
+    if(af_trace) fprintf(af_trace,"aff_write_ignore(%p,%zd)\n",af,bytes);
 
     /* First write the ignore */
     r = aff_write_ignore2(af,bytes);
@@ -98,9 +101,9 @@ static int aff_write_ignore(AFFILE *af,unsigned long bytes)
  * This is the only place where a segment actually gets written
  */
 
-int aff_write_seg(AFFILE *af, const char *segname,unsigned long arg,const u_char *data,unsigned int datalen)
+int aff_write_seg(AFFILE *af, const char *segname,unsigned long arg,const u_char *data,size_t datalen)
 {
-    //printf("aff_write_seg(%s) af->aseg=%p\n",segname,af->aseg);
+    if(af_trace) fprintf(af_trace,"aff_write_seg(%p,%s,%lu,%p,len=%zu)\n",af,segname,arg,data,datalen);
 
     struct af_segment_head segh;
     struct af_segment_tail segt;
@@ -126,12 +129,11 @@ int aff_write_seg(AFFILE *af, const char *segname,unsigned long arg,const u_char
 
     strcpy(segt.magic,AF_SEGTAIL);
     segt.segment_len = htonl(sizeof(segh)+segname_len + datalen + sizeof(segt));
-    aff_toc_insert(af,segname,ftello(af->aseg));
+    aff_toc_update(af,segname,ftello(af->aseg),datalen);
 
-#ifdef DEBUG   
-    fprintf(stderr,"aff_write_segv: putting segment %s (datalen=%d) offset=%qd\n",
-	    segname,datalen,ftello(af->aseg));
-#endif
+    
+    if(af_trace) fprintf(af_trace,"aff_write_seg: putting segment %s (datalen=%zd) offset=%"PRId64"\n",
+			 segname,datalen,ftello(af->aseg));
 
     if(fwrite(&segh,sizeof(segh),1,af->aseg)!=1) return -10;
     if(fwrite(segname,1,segname_len,af->aseg)!=segname_len) return -11;
@@ -148,50 +150,49 @@ int aff_write_seg(AFFILE *af, const char *segname,unsigned long arg,const u_char
 
 /* aff_get_segment:
  * Get the named segment, using the toc cache.
- * If there is no cache, 
- *  -  If the requested segment is the next segment, just get it.
- *  -  Otherwise go to the beginning and scan for it.
  */
 
 static int aff_get_seg(AFFILE *af,const char *name,
 		       unsigned long *arg,unsigned char *data,size_t *datalen)
 {
-    char next[AF_MAX_NAME_LEN];
-    int first = 1;
-    size_t segsize = 0;
+    if(af_trace) fprintf(af_trace,"aff_get_seg(%p,%s,arg=%p,data=%p,datalen=%p)\n",af,name,arg,data,datalen);
 
+    char next[AF_MAX_NAME_LEN];
 
     /* If the segment is in the directory, then seek the file to that location.
      * Otherwise, we'll probe the next segment, and if it is not there,
      * we will rewind to the beginning and go to the end.
      */
     struct aff_toc_mem *adm = aff_toc(af,name);
+    if(!adm) return -1;
 
-    if(adm){
-	fseeko(af->aseg,adm->offset,SEEK_SET);
-	int ret = aff_get_next_seg(af,next,sizeof(next),arg,data,datalen);
-	if(strcmp(next,name)) printf("next=%s name=%s\n",next,name);
-	assert(strcmp(next,name)==0);	// hopefully this is what they asked for
-	return ret;
-    }
-    return -1;				// couldn't find segment
+    fseeko(af->aseg,adm->offset,SEEK_SET);
+    int ret = aff_get_next_seg(af,next,sizeof(next),arg,data,datalen);
+    assert(strcmp(next,name)==0);	// hopefully this is what they asked for
+    return ret;
 }
 
 
 
-/****************************************************************
- *** Get the next segment.
- *** data must not be freed.
- *** *datalen_ is size of data; *datalen_ is modified to amount of segment
- *** if no data is specified, then just seek past it...
- *** Returns 0 on success, 
- *** -1 on end of file. (AF_ERROR_EOF)
- *** -2 if *data is not large enough to hold the segment (AF_ERROR_DATASMALL)
- *** -3 af file is corrupt; no tail (AF_ERROR_TAIL)
- ****************************************************************/
+/**
+ * Get the next segment.
+ * @param af          - The AFF file pointer
+ * @param segname     - Array to hold the name of the segment.
+ * @param segname_len - Available space in the segname array.
+ * @param arg         - pointer to the arg
+ * @param data        - pointer to the data
+ * @param datalen_    - length of the data_ array. If *datalen_==0, set to the length of the data.
+ * 
+ * @return
+ *    0 =  success.
+ *  -1  = end of file. (AF_ERROR_EOF)
+ *  -2  = *data is not large enough to hold the segment (AF_ERROR_DATASMALL)
+ *  -3  = af file is corrupt; no tail (AF_ERROR_TAIL)
+ */
 static int aff_get_next_seg(AFFILE *af,char *segname,size_t segname_len,unsigned long *arg,
 			unsigned char *data,size_t *datalen_)
 {
+    if(af_trace) fprintf(af_trace,"aff_get_next_seg()\n");
     if(!af->aseg){
 	snprintf(af->error_str,sizeof(af->error_str),"af_get_next_segv only works with aff files");
 	return AF_ERROR_INVALID_ARG;
@@ -242,8 +243,6 @@ static int aff_get_next_seg(AFFILE *af,char *segname,size_t segname_len,unsigned
 	+ strlen(segname)
 	+ data_len + sizeof(struct af_segment_tail);
 
-    off_t newpos = ftello(af->aseg);
-
     if(strcmp(segt.magic,AF_SEGTAIL)!=0){
 	snprintf(af->error_str,sizeof(af->error_str),"af_get_next_segv: AF file is truncated (AF_ERROR_TAIL).");
 	fseeko(af->aseg,start,SEEK_SET); // go back to last good position
@@ -261,12 +260,14 @@ static int aff_get_next_seg(AFFILE *af,char *segname,size_t segname_len,unsigned
 
 static int aff_rewind_seg(AFFILE *af)
 {
+    if(af_trace) fprintf(af_trace,"aff_rewind_seg()\n");
     fseeko(af->aseg,sizeof(struct af_head),SEEK_SET); // go to the beginning
     return 0;
 }
 
 
-/* Removes the last segment of an AFF file if it is blank */
+/* Removes the last segment of an AFF file if it is blank.
+ * @return 0 for success, -1 for error */
 int af_truncate_blank(AFFILE *af)
 {
     uint64_t last_loc = ftello(af->aseg);	// remember where we are
@@ -277,7 +278,7 @@ int af_truncate_blank(AFFILE *af)
 	    if(next_segment_name[0]==0){
 		/* Remove it */
 		fflush(af->aseg);
-		ftruncate(fileno(af->aseg),backspace_loc);
+		if(ftruncate(fileno(af->aseg),backspace_loc)<0) return -1;
 		return 0;
 	    }
 	}
@@ -309,14 +310,13 @@ static int aff_update_seg(AFFILE *af, const char *name,
      * remember the location and size of the AF_IGNORE segment that
      * has the smallest size that is >= strlen(name)+vallen
      */
-    size_t size_needed = strlen(name)+vallen+aff_ignore_overhead();
+    size_t size_needed = vallen+aff_segment_overhead(name);
     size_t size_closest = 0;
     uint64_t         loc_closest = 0;
     struct aff_toc_mem *adm = aff_toc(af,name);
        
-#ifdef DEBUG
-    fprintf(stderr,"aff_update_seg(name=%s,arg=%lu,vallen=%u)\n",name,arg,vallen);
-#endif
+    if(af_trace) fprintf(af_trace,"aff_update_seg(name=%s,arg=%lu,vallen=%u)\n",name,arg,vallen);
+
 
     if(adm){
 	/* Segment is in the TOC; seek to it */
@@ -386,7 +386,7 @@ static int aff_update_seg(AFFILE *af, const char *name,
 	fseeko(af->aseg,loc_closest,SEEK_SET); // move to the location
 	aff_write_seg(af,name,arg,value,vallen); // write the new segment
 	
-	int newsize = size_closest - vallen - aff_ignore_overhead() - strlen(name);
+	size_t newsize = size_closest - vallen - aff_segment_overhead(0) - strlen(name);
 	aff_write_ignore(af,newsize); // write the smaller ignore
 	return 0;
     }
@@ -411,7 +411,11 @@ static int aff_update_seg(AFFILE *af, const char *name,
 
 static int aff_del_seg(AFFILE *af,const char *segname)
 {
-    aff_toc_del(af,segname);		// remove it from the directory
+    if(af_trace) fprintf(af_trace,"aff_del_seg(%p,%s)\n",af,segname);
+
+    if(aff_toc_del(af,segname)){	// if del fails
+	return 0;			// it's not present.
+    }
 
     /* Find out if the last segment is the one we are deleting;
      * If so, we can just truncate the file.
@@ -421,7 +425,7 @@ static int aff_del_seg(AFFILE *af,const char *segname)
     af_last_seg(af,last_segname,sizeof(last_segname),&last_pos);
     if(strcmp(segname,last_segname)==0){
 	fflush(af->aseg);		// flush any ouput
-	ftruncate(fileno(af->aseg),last_pos); // make the file shorter
+	if(ftruncate(fileno(af->aseg),last_pos)) return -1; // make the file shorter
 	return 0;
     }
 
@@ -464,16 +468,6 @@ static int aff_create(AFFILE *af)
 }
 
 
-static bool last4_is_aff(const char *filename)
-{
-    if(strlen(filename)>4){
-	const char *last4 = filename+strlen(filename)-4;
-	if(strcasecmp(last4,".aff")==0) return 1;
-    }
-    return 0;
-}
-
-
 /****************************************************************
  *** VNODE implementation functions
  ****************************************************************/
@@ -518,7 +512,7 @@ static int aff_identify_file(const char *filename,int exists)
 	/* If it is a zero-length file and the file extension ends AFF,
 	 * then let it be an AFF file...
 	 */
-	if(r==0 && last4_is_aff(filename)) return 1;
+	if(r==0 && af_ext_is(filename,"aff")) return 1;
 	return 0;			// must not be an aff file
     }
     return 0;
@@ -536,13 +530,14 @@ static int aff_open(AFFILE *af)
 	return -1;			
     }
 
-    /* Lock the file */
+    /* Lock the file if writing */
 #ifdef HAVE_FLOCK
-    int lockmode = LOCK_SH;		// default
-    if((af->openflags & O_ACCMODE)==O_RDWR) lockmode = LOCK_EX; // there can be only one
-    if(flock(fd,lockmode)){
-	close(fd);
-	return -1;			// couldn't get lock
+    if(af->openflags & O_RDWR){
+	int lockmode = LOCK_SH;		// default
+	if((af->openflags & O_ACCMODE)==O_RDWR) lockmode = LOCK_EX; // there can be only one
+	if(flock(fd,lockmode)){
+	    warn("Cannot exclusively lock %s:",af->fname);
+	}
     }
 #endif
 
@@ -604,6 +599,7 @@ static int aff_open(AFFILE *af)
  */
 static int aff_close(AFFILE *af)
 {
+    aff_toc_free(af);
     fclose(af->aseg);
     return 0;
 }
@@ -617,17 +613,21 @@ static int aff_vstat(AFFILE *af,struct af_vnode_info *vni)
     vni->supports_compression = 1;
     vni->has_pages            = 1;
     vni->supports_metadata    = 1;
+    vni->cannot_decrypt       = af_cannot_decrypt(af) ? 1 : 0;
 
     /* Check for an encrypted page */
     if(af->toc){
 	for(int i=0;i<af->toc_count;i++){
 	    if(af->toc[i].name){
+		bool is_page = false;
 		vni->segment_count_total++;
 		if(af_segname_page_number(af->toc[i].name)>=0){
 		    vni->page_count_total++;
+		    is_page = true;
 		}
 		if(af_is_encrypted_segment(af->toc[i].name)){
 		    vni->segment_count_encrypted++;
+		    if(is_page) vni->page_count_encrypted++;
 		}
 		if(af_is_signature_segment(af->toc[i].name)){
 		    vni->segment_count_signed++;
