@@ -62,7 +62,7 @@ static int isdigit(char ch)
 
 /* split raw file implementation with optional metadata support */
 struct split_raw_private {
-    uint   num_raw_files;	// number of raw files
+    u_int   num_raw_files;	// number of raw files
     int    *fds;		// array of file descriptors for each open raw file
     uint64_t *pos;		// where we are in each file
     char *first_raw_fname;    /* The filename of the first raw file. */
@@ -77,24 +77,13 @@ static inline struct split_raw_private *SPLIT_RAW_PRIVATE(AFFILE *af)
     return (struct split_raw_private *)(af->vnodeprivate);
 }
 
-static inline const char *last4(const char *filename)
-{
-  if(strlen(filename)>4) {
-    const char *l = filename+strlen(filename)-4;
-    return l;
-  }
-  return "";
-}
-
 /* Return 1 if a file is a split raw file... */
 static int split_raw_identify_file(const char *filename,int exists)
 {
     if(exists && access(filename,R_OK)!=0) return 0;	// needs to exist and it doesn't
-    const char *l4 = last4 (filename);
-
-    return (!strcmp (l4, ".000"))
-	|| (!strcmp (l4, ".001"))
-	|| (!strcasecmp (l4, ".aaa"));
+    return af_ext_is(filename,"000") ||
+	af_ext_is(filename,"001") ||
+	af_ext_is(filename,"aaa");
 }
 
 /* split_raw_close:
@@ -436,7 +425,7 @@ int split_raw_write_internal2(AFFILE *af, unsigned char *buf, uint64_t pos,size_
 	    char z = 0;
 
 	    lseek(srp->fds[i],c3-1,SEEK_CUR);
-	    write(srp->fds[i],&z,1);
+	    if(write(srp->fds[i],&z,1)!=1) return -1; // failure
 	    c4 = c3;
 	}
 
@@ -586,7 +575,7 @@ static int split_raw_update_seg(AFFILE *af, const char *name,
 	
     uint64_t pos = page_num * af->image_pagesize; // where we are to start reading
     int written = split_raw_write(af, (unsigned char *)value, pos,vallen);
-    if(written==vallen) return 0;	// success
+    if(written==(int)vallen) return 0;	// success
     return -1;
 }
 
