@@ -327,11 +327,21 @@ int  compare_aff_data_segments(char *title,AFFILE *af1,AFFILE *af2,int64_t pagen
 
     uint64_t start_sector_number = (pagenum * data1_len) / af1->image_sectorsize;
 
+    /* Find the size of each page, then get the page */
+    if(af_get_page(af1,pagenum,0,&data1_len)<0)
+	err(1,"Cannot read page %"I64d" size from %s\n",pagenum,af_filename(af1));
     if(af_get_page(af1,pagenum,data1,&data1_len)<0)
-	err(1,"Cannot read page %"I64d" from %s\n",pagenum,af_filename(af1));
+	err(1,"Cannot read page %"I64d" from %s",pagenum,af_filename(af1));
 
+    if(af_get_page(af2,pagenum,0,&data2_len)<0)
+	err(1,"Cannot read page %"I64d" size from %s\n",pagenum,af_filename(af2));
     if(af_get_page(af2,pagenum,data2,&data2_len)<0)
-	err(1,"Cannot read page %"I64d" from %s\n",pagenum,af_filename(af2));
+	err(1,"Cannot read page %"I64d" from %s",pagenum,af_filename(af2));
+
+    if(data1_len != data2_len){
+	printf("page %"I64d" size %zd != size %zd\n",pagenum,data1_len,data2_len);
+	return 1;
+    }
 
     /* Now look at the pages sector-by-sector. */
     int af1_bad=0;
@@ -529,7 +539,7 @@ int compare_aff_aff(const char *file1,const char *file2)
 	no_data_segments = true;
     }
 
-    if(af1->image_pagesize != af2->image_pagesize){
+    if(af1->image_sectorsize != af2->image_sectorsize){
 	fprintf(stderr,"Currently, %s requires that both images have the "
 		"same image sectorsize.\n"
 		"sectorsize(%s)=%ld\n"
@@ -610,7 +620,7 @@ int compare_aff_aff(const char *file1,const char *file2)
     int highest_page = -1;
     /* Scan for the lowest and highest numbers */
     for(vector<string>::iterator i = segs.begin();i != segs.end(); i++){
-	int num = af_segname_page_number(i->c_str());
+	int64_t num = af_segname_page_number(i->c_str());
 	if(num!=-1){
 	    if(num<lowest_page ||lowest_page==-1)  lowest_page = num;
 	    if(num>highest_page||highest_page==-1) highest_page = num;
@@ -647,7 +657,7 @@ int compare_aff_aff(const char *file1,const char *file2)
 	    }
 	    
 	    for(vector<string>::iterator i = segs.begin();i != segs.end();i++){
-		int num = af_segname_page_number(i->c_str());
+		int64_t num = af_segname_page_number(i->c_str());
 		if(num==-1){
 		    int r = compare_aff_metadata_segments(mode_title, af1,af2,
 							  i->c_str(),mode);
@@ -657,20 +667,22 @@ int compare_aff_aff(const char *file1,const char *file2)
 	}
     }
 
-    if(opt_all) puts("Inspecting data...");
-    for(int mode=1;mode<=3;mode++){
-	char mode_title[1024];
-	switch(mode){
-	case 1: snprintf(mode_title,sizeof(mode_title),
-			 "  Pages only in %s:\n", af_filename(af1));break;
-	case 2: snprintf(mode_title,sizeof(mode_title),
-			 "  Pages only in %s:\n", af_filename(af2));break;
-	case 3: snprintf(mode_title,sizeof(mode_title),"  Pages in both files:\n");break;
-	}
+    if(no_data_segments==false){
+	if(opt_all) puts("Inspecting data...");
+	for(int mode=1;mode<=3;mode++){
+	    char mode_title[1024];
+	    switch(mode){
+	    case 1: snprintf(mode_title,sizeof(mode_title),
+			     "  Pages only in %s:\n", af_filename(af1));break;
+	    case 2: snprintf(mode_title,sizeof(mode_title),
+			     "  Pages only in %s:\n", af_filename(af2));break;
+	    case 3: snprintf(mode_title,sizeof(mode_title),"  Pages in both files:\n");break;
+	    }
 	    
-	for(int i=lowest_page;i<=highest_page;i++){
-	    int r = compare_aff_data_segments(mode_title,af1,af2,i,mode);
-	    if(r!=0) ret = r;
+	    for(int i=lowest_page;i<=highest_page;i++){
+		int r = compare_aff_data_segments(mode_title,af1,af2,i,mode);
+		if(r!=0) ret = r;
+	    }
 	}
     }
     current_source = 0;
