@@ -51,6 +51,7 @@
 #include "affconfig.h"
 #include "afflib.h"
 #include "afflib_i.h"			// we do enough mucking, we need the internal version
+#include "utils.h"
 
 #include <openssl/md5.h>
 #include <openssl/sha.h>
@@ -67,18 +68,18 @@
 #include <term.h>
 #endif
 
-#ifdef HAVE_NCURSES_TERM_H
-#include <ncurses/term.h>
+
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
 #endif
 
+#include <sys/stat.h>
+#include <string>
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-#ifdef linux
-#include <sys/time.h>
-#endif
-
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
@@ -89,20 +90,20 @@ const char *progname = "afconvert";
 
 int	image_pagesize = 16*1024*1024;	// default seg size --- 16MB
 int	opt_compression_alg	= AF_COMPRESSION_ALG_ZLIB;
-int	opt_compress_level  = AF_COMPRESSION_DEFAULT;
-int64_t	 bytes_to_convert = 0;
-int	opt_batch	= 1;
-int	opt_zap = 0;
-int	opt_quiet	= 0;
-int	opt_nozprobe	= 0;
+int	opt_compress_level	= AF_COMPRESSION_DEFAULT;
+int64_t	 bytes_to_convert	= 0;
+int	opt_batch = 1;
+int	opt_zap   = 0;
+int	opt_quiet = 0;
 int	opt_write_raw		= 0;		// output
+int	opt_probe_compressed = 1;		// probe for compressed files
 const char	*opt_write_raw_ext	= "raw";
 const char	*opt_outdir     = 0;
 const char	*opt_aff_ext    = "aff";
 int64_t	opt_maxsize     = 0;
 int	opt_yes		= 0;
 int	opt_debug       = 0;
-char    *command_line = 0;	// what is typed
+std::string	command_line;
 
 
 char *append(char *base,const char *str)
@@ -199,7 +200,7 @@ int yesno(const char *statement,const char *question,const char *affirmative)
     do {
 	printf("%s [y/n]: ",question);
 	memset(buf,0,sizeof(buf));
-	fgets(buf,sizeof(buf)-1,stdin);
+	if(fgets(buf,sizeof(buf)-1,stdin)==0) return 0;
 	if(buf[0]=='y' || buf[0]=='Y'){
 	    printf("%s.\n",affirmative);
 	    return 1;
@@ -229,27 +230,28 @@ int yesno(const char *statement,const char *question,const char *affirmative)
 
 using namespace std;
 
+/** Do the conversion.
+ * return 0 if success, code if fail.
+ */
 int convert(const char *infile,char *outfile)
 {
+
     if(opt_debug) fprintf(stderr,"convert(%s,%s)\n",infile,outfile);
-    if(access(infile,F_OK)!=0){
-	err(1,"%s",infile);			// file does not exist?
-    }
 
     if(infile && outfile && strcmp(infile,outfile)==0){
 	errx(1,"Can't convert a file to itself\n");
     }
 
-    if(!opt_quiet) printf("convert %s --> %s\n",infile,outfile);
-
     /****************************************************************
      *** Open Input
      ****************************************************************/
 
     AFFILE *a_in = 0;			// input file, if aff
+
 #ifdef UNIX
     /* Check to see if it is a gzip file... */
-    if(probe_gzip(infile)
+    if(opt_probe_compressed
+       && probe_gzip(infile)
        && yesno("infile looks like a gzip file","Uncompress it","Uncompressing")){
 	/* Open with a subprocess. We will need to use zlib when we move to Windows. */
 	if(af_hasmeta(infile)) return -1;	// don't covert with shell metacharacters
@@ -260,6 +262,7 @@ int convert(const char *infile,char *outfile)
 
     /* Check to see if it is a bzip2 file... */
     if(!a_in
+       && opt_probe_compressed
        && probe_bzip2(infile)
        && yesno("infile looks like a bzip2 file","Uncompress it","Uncompressing")){
 	/* Open with a subprocess. We will need to use bzip2zlib when we move to Windows. */
@@ -282,6 +285,13 @@ int convert(const char *infile,char *outfile)
 	}
     }
     
+    const char *ain_fn = af_filename(a_in);
+    struct stat si;
+    memset((char *)&si,0,sizeof(si));
+    if(ain_fn && stat(ain_fn,&si)){
+	warn("Cannot stat %s",ain_fn);
+    }
+
 
     /****************************************************************
      *** Open Ouptut
@@ -293,12 +303,12 @@ int convert(const char *infile,char *outfile)
     AFFILE *a_out = 0;			// output file, if aff or raw...
     if(access(outfile,F_OK)==0){
 	/* If outfile is a device, ask user... */
-	struct stat sb;
-	if(stat(outfile,&sb)){
+	struct stat so;
+	if(stat(outfile,&so)){
 	    err(1,"%s exists but can't be stat?",outfile);
 	}
-	if((sb.st_mode & S_IFMT)==S_IFCHR ||
-	   (sb.st_mode & S_IFMT)==S_IFBLK){
+	if((so.st_mode & S_IFMT)==S_IFCHR ||
+	   (so.st_mode & S_IFMT)==S_IFBLK){
 	    char buf[1024];
 	    snprintf(buf,sizeof(buf),"%s is a raw device.\n",outfile);
 	    if(yesno(buf,"Overwrite raw device?","yes")){
@@ -326,7 +336,6 @@ int convert(const char *infile,char *outfile)
 
  doit:;
 
-
     if(opt_write_raw){
 	/* Easy way to make a raw output is to reopen an existing output file... */
 	FILE *f = fopen(outfile,"w+b");
@@ -344,19 +353,18 @@ int convert(const char *infile,char *outfile)
 
     }
     if(a_out == 0) af_err(1,"af_open: %s",outfile);
-    af_update_seg(a_out,AF_ACQUISITION_COMMAND_LINE,0,(const u_char *)command_line,strlen(command_line));
+
+    if(!opt_quiet) printf("convert %s --> %s\n",infile,outfile);
+
+    af_update_seg(a_out,AF_ACQUISITION_COMMAND_LINE,0,
+		  (const u_char *)command_line.c_str(),
+		  command_line.size());
 
     /****************************************************************
      *** Set up the AFF file (assuming it's an aff file)
      *** stuff that we keep at the beginning of the file...
      ****************************************************************/
 
-    unsigned char md5_buf[16];
-    unsigned char sha1_buf[20];
-
-    memset(md5_buf,0,sizeof(md5_buf));
-    memset(sha1_buf,0,sizeof(sha1_buf));
-
     MD5_CTX md5;
     MD5_Init(&md5);
 
@@ -380,10 +388,6 @@ int convert(const char *infile,char *outfile)
 	    af_enable_compression(a_out,0,0);
 	}
     }
-    if(vni.supports_metadata){
-	af_update_seg(a_out,AF_MD5,0,md5_buf,16);
-	af_update_seg(a_out,AF_SHA1,0,sha1_buf,20);
-    }
 
     /* Get a list of all the metadata segments and the pages
      * (if this is a raw file, then the vnode raw driver will give us those segments)
@@ -436,7 +440,6 @@ int convert(const char *infile,char *outfile)
 	}
     }
 	
-
     /* Now sort the pages and copy them over. If there is no break,
      * we can compute the hashes...
      */
@@ -449,6 +452,7 @@ int convert(const char *infile,char *outfile)
 
     bool copy_by_pages = af_has_pages(a_in);
 
+    unsigned char *data = (unsigned char *)malloc(image_pagesize);
     if(copy_by_pages){
 	/* Copy over data one page at a time */
 	for(vector<int64_t>::iterator i = pages.begin(); i != pages.end(); i++){
@@ -457,7 +461,6 @@ int convert(const char *infile,char *outfile)
 	    
 	    if(!opt_quiet) printf("Converting page %"I64d" of %"I64d"\r",pagenum,highest_pagenum);fflush(stdout);
 	    
-	    unsigned char *data = (unsigned char *)malloc(image_pagesize);
 	    size_t data_len = image_pagesize;
 	    if(af_get_page(a_in,pagenum,data,&data_len)){
 		err(1,"af_get_page(file=%s,page=%"I64d")",
@@ -470,18 +473,21 @@ int convert(const char *infile,char *outfile)
 	    
 	    if(pagenum != prev_pagenum + 1) hash_valid = false;
 	    
-	    if(hash_valid){
+	    if(hash_valid && vni.supports_metadata){
 		MD5_Update(&md5,data,data_len);
 		SHA1_Update(&sha,data,data_len);
 		prev_pagenum = pagenum;
 	    }
-	    free(data); data = 0;
 	    last_byte_in_image = (int64_t)image_pagesize * pagenum + (int64_t)data_len;
 	    total_bytes_converted += data_len;
 	}
+	/* Go back and update the image size (necessary since I have been writing page-by-page) */
+	if(af_update_segq(a_out,AF_IMAGESIZE,last_byte_in_image)
+	   && errno!=ENOTSUP){
+	    err(1,"Could not upate AF_IMAGESIZE");
+	}
     } else {
 	/* No page support; Copy from beginning to end */
-	unsigned char *data = (unsigned char *)malloc(image_pagesize);
 	while(!af_eof(a_in)){
 	    int data_len = af_read(a_in,data,image_pagesize);
 	    if(data_len>0){
@@ -493,8 +499,10 @@ int convert(const char *infile,char *outfile)
 		if(af_write(a_out,data,data_len)!=data_len){
 		    err(1,"af_write");
 		}
-		MD5_Update(&md5,data,data_len);
-		SHA1_Update(&sha,data,data_len);
+		if(vni.supports_metadata){
+		    MD5_Update(&md5,data,data_len);
+		    SHA1_Update(&sha,data,data_len);
+		}
 	    }
 	    if(data_len<0) err(1,"af_read");
 	    if(data_len==0){
@@ -504,15 +512,15 @@ int convert(const char *infile,char *outfile)
 	    last_byte_in_image += data_len;
 	    total_bytes_converted += data_len;
 	}
-	free(data);
     }
+    free(data);
     if(!opt_quiet) printf("\n");
 
-
     /* Write out the new hash if it is valid */
-    if(hash_valid){
-	MD5_Final(md5_buf,&md5);
+    if(hash_valid && vni.supports_metadata){
+	u_char md5_buf[32],sha1_buf[40];
 	char buf[256];
+	MD5_Final(md5_buf,&md5);
 	if(af_update_seg(a_out,AF_MD5,0,md5_buf,16) && errno!=ENOTSUP){
 	    err(1,"Could not update AF_MD5");
 	}
@@ -525,16 +533,9 @@ int convert(const char *infile,char *outfile)
 	if(!opt_quiet) printf("sha1: %s\n",af_hexbuf(buf,sizeof(buf),sha1_buf,20,1));
     }
 
-    /* Go back and update the image size */
-    if(af_update_segq(a_out,AF_IMAGESIZE,last_byte_in_image)
-       && errno!=ENOTSUP){
-	err(1,"Could not upate AF_IMAGESIZE");
-    }
-
     /* Finish the hash calculations and write to the db */
     if(!opt_quiet){
 	printf("bytes converted: %"I64d" \n",total_bytes_converted);
-
 	/* If the vnode implementation tracked segments written, report it. */
 	if(a_out->pages_written || a_out->pages_compressed){
 	    printf("Total pages: %"I64u"  (%"I64u" compressed)\n",
@@ -542,18 +543,43 @@ int convert(const char *infile,char *outfile)
 	}
     }
 
-    af_make_gid(a_out);			// make an AF_IMAGE_GID if it doesn't exist
+    if(vni.supports_metadata){
+	/* make an AF_IMAGE_GID if it doesn't exist */
+	af_make_gid(a_out);		
+	af_set_acquisition_date(a_out,si.st_mtime);
+    }
 
-    if(af_close(a_in)) err(1,"af_close(a_in)");
+    /* Make a copy of the a_out filename if we can get it */
+    char *a_out_fn=0;			 // output filename, to remember for utimes
+    const char *a_ = af_filename(a_out); // remember the output filename
+    if(a_){
+	a_out_fn = strdup(a_);	// make a copy of it
+    }
     if(af_close(a_out)) err(1,"af_close(a_out)");
 
     if(!opt_quiet){
 	printf("Conversion finished.\n");
+	if(af_cannot_decrypt(a_in)){
+	    printf("*** encrypted pages are present which could not be decrypted ***\n");
+	}
 	printf("\n\n");
     }
+    if(af_close(a_in)) err(1,"af_close(a_in)");
 
     /* Set the utime on the resulting file if we can stat it */
-    exit(0);
+    struct timeval times[2];
+    
+    memset(times,0,sizeof(times));
+    times[0].tv_sec = si.st_atime;
+    times[1].tv_sec = si.st_mtime;
+#ifdef HAVE_UTIMES
+    if(a_out_fn){
+	if(utimes(a_out_fn,times)) warn("utimes(%s):",outfile);
+	free(a_out_fn);
+	a_out_fn = 0;
+    }
+#endif    
+    return(0);
 }
 
 
@@ -592,20 +618,7 @@ int main(int argc,char **argv)
     char *outfile = 0;
     int ch;
 
-#ifdef HAVE_LIBNCURSES
-    setupterm((char *)0,1,(int *)0);
-#endif
-
-    /* Figure out the command line that was used to run this program.
-     * This information will be recorded in the AFF file.
-     */
-    command_line = (char *)malloc(1);
-    command_line[0] = 0;
-    for(int i=0;i<argc;i++){
-	if(i>0) command_line = append(command_line," ");
-	command_line = append(command_line,argv[i]);
-    }
-
+    command_line = aff::command_line(argc,argv);
     while ((ch = getopt(argc, argv, "a:e:Lo:zqrs:xX:Zh?M:O::ydV")) != -1) {
 	switch (ch) {
 	case 'a':
@@ -640,7 +653,7 @@ int main(int argc,char **argv)
 	    opt_compress_level = atoi(optarg);
 	    break;
 	case 'Z':
-	    opt_nozprobe++;
+	    opt_probe_compressed = 0;
 	    break;
 	case 'y':
 	    opt_yes = 1;
@@ -674,12 +687,17 @@ int main(int argc,char **argv)
     }
 
     if(outfile){
-	convert(*argv,outfile);
-	return 0;			// only can convert a single file
+	return convert(*argv,outfile);
     }
 
+    /* Check for "-o filename" at the end of the command line... */
+    if(argc==3 && !strcmp(argv[1],"-o")){
+	return convert(argv[0],argv[2]);
+    }
+
+    /* Convert each file*/
+
     while(*argv){
-	/* Come up with output file name */
 	char outfile[MAXPATHLEN+1];
 	memset(outfile,0,sizeof(outfile));
 
@@ -688,45 +706,38 @@ int main(int argc,char **argv)
 	argv++;
 	argc--;
 
-	/* Check for "-o filename" to specify the output arg... */
-	if(argc>=2 && strcmp(*argv,"-o")==0){
-	    strlcpy(outfile,argv[1],sizeof(outfile));
-	    argv += 2;
-	    argc -= 2;
+	/* Copy over the filename and change the extension */
+	strlcpy(outfile,infile,sizeof(outfile));
+	char *cc = strrchr(outfile,'.'); // to strip off extension
+	if(cc){
+	    /* Found an extension; copy over mine. */
+	    strlcpy(cc+1,ext,sizeof(outfile)-(cc-outfile));
 	}
 	else {
-
-	    /* Copy over the filename and change the extension */
-	    strlcpy(outfile,infile,sizeof(outfile));
-	    char *cc = strrchr(outfile,'.'); // to strip off extension
+	    /* No extension; make one */
+	    strlcat(outfile,".",sizeof(outfile));
+	    strlcat(outfile,ext,sizeof(outfile));
+	}
+	
+	/* The user might want us to put things
+	 * in a different directory. Pull off the filename...
+	 */
+	if(opt_outdir){
+	    cc = strrchr(outfile,'/');
+	    char filename[PATH_MAX];
 	    if(cc){
-		/* Found an extension; copy over mine. */
-		strlcpy(cc+1,ext,sizeof(outfile)-(cc-outfile));
+		strlcpy(filename,cc+1,sizeof(filename));	// just the filename
 	    }
-	    else {
-		/* No extension; make one */
-		strlcat(outfile,".",sizeof(outfile));
-		strlcat(outfile,ext,sizeof(outfile));
+	    else{
+		strlcpy(filename,outfile,sizeof(filename));	// the outfile is the filename
 	    }
-
-	    /* The user might want us to put things
-	     * in a different directory. Pull off the filename...
-	     */
-	    if(opt_outdir){
-		cc = strrchr(outfile,'/');
-		char filename[PATH_MAX];
-		if(cc){
-		    strlcpy(filename,cc+1,sizeof(filename));	// just the filename
-		}
-		else{
-		    strlcpy(filename,outfile,sizeof(filename));	// the outfile is the filename
-		}
-		strlcpy(outfile,opt_outdir,sizeof(outfile));
-		strlcat(outfile,"/",sizeof(outfile));
-		strlcat(outfile,filename,sizeof(outfile));
-	    }
+	    strlcpy(outfile,opt_outdir,sizeof(outfile));
+	    strlcat(outfile,"/",sizeof(outfile));
+	    strlcat(outfile,filename,sizeof(outfile));
+	}
+	if(convert(infile,outfile)){
+	    exit(1);
 	}
-	convert(infile,outfile);
     }
     exit(0);
 }
