@@ -53,7 +53,11 @@
 const char *progname = "afcrypto";
 #define DEFAULT_PASSPHRASE_FILE ".affpassphrase"
 int opt_debug = 0;
+int opt_verbose = 0;
+int opt_just_print_encrypted_count = 0;
+int opt_just_print_unencrypted_count = 0;
 char *opt_unsealing_private_key_file= 0;
+int opt_xml = 0;
 
 void change_passphrase(const char *fn,const char *old_passphrase,const char *new_passphrase)
 {
@@ -78,7 +82,7 @@ void get_and_change_passphrase(const char *fn)
     memset(new_passphrase,0,sizeof(new_passphrase));
 
     printf("Enter old passphrase: ");
-    fgets(old_passphrase,sizeof(old_passphrase),stdin);
+    if(fgets(old_passphrase,sizeof(old_passphrase),stdin)==0) return;
     char *cc = strchr(old_passphrase,'\n');if(cc) *cc='\000';
 
     /* See if this passphrase works*/
@@ -91,7 +95,7 @@ void get_and_change_passphrase(const char *fn)
     af_close(af);
 
     printf("Enter new passphrase: ");
-    fgets(new_passphrase,sizeof(new_passphrase),stdin);
+    if(fgets(new_passphrase,sizeof(new_passphrase),stdin)==0) return;
     cc = strchr(new_passphrase,'\n');if(cc) *cc='\000';
     change_passphrase(fn,old_passphrase,new_passphrase);
 }
@@ -102,7 +106,13 @@ void usage()
     printf("usage: afcrypto [options] filename.aff [filename2.aff ... ]\n");
     printf("   prints if each file is encrypted or not.\n");
     printf("options:\n");
+    printf("    -x      --- output in XML\n");
+    printf("    -j      --- Just print the number of encrypted segments\n");
+    printf("    -J      --- Just print the number of unencrypted segments\n");
+
+    printf("\nData conversion options:\n");
     printf("    -e      --- encrypt the unencrypted non-signature segments\n");
+    printf("    -d      --- decrypt the encrypted non-signature segments\n");
     printf("    -r      --- change passphrase (take old and new from stdin)\n");
     printf("    -O old  --- specify old passphrase\n");
     printf("    -N new  --- specify new passphrase\n");
@@ -122,7 +132,8 @@ void usage()
 
     printf("\nDebugging:\n");
     printf("    -V      --- Just print the version number and exit.\n");
-    printf("    -d      --- debug; print out each key as it is tried\n");
+    printf("    -D      --- debug; print out each key as it is tried\n");
+    printf("    -l      --- List the installed hash and encryption algorithms \n");
     printf("Note: This program ignores the environment variables:\n");
     puts(AFFLIB_PASSPHRASE);
     puts(AFFLIB_PASSPHRASE_FILE);
@@ -137,12 +148,11 @@ char  *check_file(AFFILE *af,const char *passphrase_file)
     char *ret = 0;
     FILE *f = fopen(passphrase_file,"r");
     if(!f) return 0;
-    int t=0;
 
     char buf[1024];
     memset(buf,0,sizeof(buf));
     while(fgets(buf,sizeof(buf)-1,f)){
-	char *cc = index(buf,'\n');
+	char *cc = strchr(buf,'\n');
 	if(cc) *cc = 0;
 	if(opt_debug){
 	    if(opt_debug) printf("checking with '%s' ... ",buf);
@@ -159,14 +169,90 @@ char  *check_file(AFFILE *af,const char *passphrase_file)
     return ret;
 }
 
+/**
+ * This will eventually decrypt non-signature segments that are
+ * encrypted
+ *
+ * @param af - the AFFILE to open
+ * @param count - The number of pages actually encrypted
+ */
+int af_decrypt_encrypted_segments(AFFILE *af, int *count, int mode)
+{
+    af_set_option(af,AF_OPTION_AUTO_ENCRYPT,0);
+    af_set_option(af,AF_OPTION_AUTO_DECRYPT,0); // turn off auto decryption
+    aff::seglist sl(af);	       // get the list of the segments
+    af_set_option(af,AF_OPTION_AUTO_DECRYPT,1); // turn auto decryption back on
+    for(aff::seglist::const_iterator si = sl.begin();si!=sl.end();si++){
+	if(opt_debug) printf(" checking segment %s",si->name.c_str());
+	if(af_is_encrypted_segment(si->name.c_str())){
+
+	    if(mode == O_RDONLY){	// if readonly, just tally
+		(*count) ++;
+		if(opt_debug) printf("  would decrypt segment\n");
+		continue;
+	    }
+
+	    /* Generate the name of the unencrypted segment */
+	    char segname[AF_MAX_NAME_LEN];
+	    strcpy(segname,si->name.c_str());
+	    char *cc = strstr(segname,AF_AES256_SUFFIX);
+	    if(!cc){
+		if(opt_debug) printf(" will not decrypt AFFKEY segments; will be deleted later.\n");
+		continue;		// something is wrong; can't find the /aes256
+	    }
+	    *cc = '\000';		// truncate off the /aes256
+
+	    /* Get the segment and put it, which will force the decryption to take place */
+	    if(opt_debug) printf("  decrypting segment\n");
+	    u_char *buf = (u_char *)malloc(si->len);
+	    if(!buf) warn("malloc(%zd) failed", si->len);
+	    else {
+		unsigned long arg;
+		size_t datalen = si->len;
+		if(af_get_seg(af,segname,&arg,buf,&datalen)){
+		    warn("Could not read segment '%s'",segname);
+		}
+		else{
+		    /* si->datalen >= datalen.
+		     * si->datalen is the length of the encrypted segment.
+		     * datalen is the length of the decrypted segment.
+		     */
+		    assert(si->len >= datalen);
+		    assert(si->arg==arg);
+		    if(af_update_seg(af,segname,arg,buf,datalen)){
+			warn("Could not decrypt segment '%s'",si->name.c_str());
+		    } else {
+			(*count) ++;
+		    }
+		}
+		free(buf);
+	    }
+	} else {
+	    if(opt_debug) printf("  not encrypted\n");
+	}
+    }
+    /* Delete the AF_AFFKEY segment */
+    if(af_get_seg(af,AF_AFFKEY,0,0,0)==0) af_del_seg(af,AF_AFFKEY);
+    /* Delete all of the EVP segments */
+    for(int i=0;;i++){
+	char segname[1024];
+	snprintf(segname,sizeof(segname),AF_AFFKEY_EVP,i);
+	if(af_get_seg(af,segname,0,0,0)!=0) break; // found the last segment
+	if(af_del_seg(af,segname)) warn("Cannot delete segment %s",segname);
+    }
+    return 0;
+}
+
+
 /**
  * Encrypts the non-signature segments that are not encrypted.
  * There is no reason to encrypt the signature segments.
  *
  * @param af - the AFFILE to open
+ * @param count - The number of pages actually encrypted
  */
 
-int af_encrypt_unencrypted_unsigned_segments(AFFILE *af)
+int af_encrypt_unencrypted_nonsignature_segments(AFFILE *af,int *count,int mode)
 {
     af_set_option(af,AF_OPTION_AUTO_DECRYPT,0);	// do not automatically decrypt
     aff::seglist sl(af);
@@ -175,10 +261,16 @@ int af_encrypt_unencrypted_unsigned_segments(AFFILE *af)
 	if(strstr(si->name.c_str(),"affkey_evp")) continue;
 	if(!af_is_encrypted_segment(si->name.c_str()) &&
 	   !af_is_signature_segment(si->name.c_str())){
+
+	    if(mode == O_RDONLY){	// if readonly, just tally
+		(*count) ++;
+		continue;
+	    }
+
 	    /* Get the segment and put it, which will force the encryption to take place */
 	    if(opt_debug) printf("  encrypting segment %s\n",si->name.c_str());
 	    u_char *buf = (u_char *)malloc(si->len);
-	    if(!buf) warn("Cannot encrypt segment '%s' --- too large (%d bytes) --- malloc failed",
+	    if(!buf) warn("Cannot encrypt segment '%s' --- too large (%zd bytes) --- malloc failed",
 			  si->name.c_str(),si->len);
 	    else {
 		unsigned long arg;
@@ -193,6 +285,7 @@ int af_encrypt_unencrypted_unsigned_segments(AFFILE *af)
 		    if(af_update_seg(af,si->name.c_str(),arg,buf,datalen)){
 			warn("Could not encrypt segment '%s'",si->name.c_str());
 		    } else {
+			(*count) ++;
 		    }
 		}
 		free(buf);
@@ -205,6 +298,16 @@ int af_encrypt_unencrypted_unsigned_segments(AFFILE *af)
     return 0;
 }
 
+void list_openssl_hashes()
+{
+    const char *digests[] = {"md5","sha1","sha256",0};
+    OpenSSL_add_all_algorithms();
+    for(int i=0;digests[i];i++){
+	printf("OpenSSL has %s: %s\n",digests[i],EVP_get_digestbyname(digests[i]) ? "YES" : "NO");
+    }
+    exit(0);
+}
+
 int main(int argc,char **argv)
 {
     int bflag, ch;
@@ -214,28 +317,41 @@ int main(int argc,char **argv)
     char *passphrase_file = 0;
     const char *progname = argv[0];
     int opt_encrypt = 0;
+    int opt_decrypt = 0;
     int opt_add_passphrase_to_public_key = 0;
     int opt_add_public_key_to_passphrase = 0;
     
     int mode = O_RDONLY;		// mode for opening AFF file
     const char **certificates = (const char **)malloc(0);
     int num_certificates = 0;
-
+    const char *envvars[] = {AFFLIB_PASSPHRASE,AFFLIB_PASSPHRASE_FILE,AFFLIB_PASSPHRASE_FD,
+		      AFFLIB_DECRYPTING_PRIVATE_KEYFILE,0};
+    for(int i=0;envvars[i];i++){
     /* Don't use auto-supplied passphrases */
-    unsetenv(AFFLIB_PASSPHRASE);
-    unsetenv(AFFLIB_PASSPHRASE_FILE);
-    unsetenv(AFFLIB_PASSPHRASE_FD);
-    unsetenv(AFFLIB_DECRYPTING_PRIVATE_KEYFILE);
-
+#ifdef HAVE_UNSETENV
+	unsetenv(envvars[i]);
+#else
+	if(getenv(envvars[i])){
+	    fprintf(stderr,"Please unset %s and restart\n",envvars[i]);
+	    exit(1);
+	}
+#endif
+    }
+    
     bflag = 0;
     int opt_change = 0;
     const char *home = getenv("HOME");
-    while ((ch = getopt(argc, argv, "reC:SAO:N:p:f:kdh?VK:")) != -1) {
+
+    while ((ch = getopt(argc, argv, "zreC:SAO:N:p:f:kdDh?VK:vljJx:")) != -1) {
 	switch (ch) {
+	case 'x': opt_xml = 1; break;
+	case 'j': opt_just_print_encrypted_count =1;break;
+	case 'J': opt_just_print_unencrypted_count =1;break;
 
 	    /* These options make the mode read-write */
-	case 'r': opt_change = 1; mode = O_RDWR; break;
+	case 'r': opt_change = 1;  mode = O_RDWR; break;
 	case 'e': opt_encrypt = 1; mode = O_RDWR; break;
+	case 'd': opt_decrypt = 1; mode = O_RDWR; break;
 	case 'S': opt_add_passphrase_to_public_key = 1; mode = O_RDWR; break;
 	case 'A': opt_add_public_key_to_passphrase = 1; mode = O_RDWR; break;
 	    /* These just set up variables */
@@ -256,7 +372,9 @@ int main(int argc,char **argv)
 	    strcat(passphrase_file,"/");
 	    strcat(passphrase_file,DEFAULT_PASSPHRASE_FILE);
 	    break;
-	case 'd': opt_debug = 1;break;
+	case 'D': opt_debug = 1;break;
+	case 'v': opt_verbose = 1;break;
+	case 'l': list_openssl_hashes(); exit(0);
 	case 'h':
 	case '?':
 	default:
@@ -269,23 +387,27 @@ int main(int argc,char **argv)
     }
     argc -= optind;
     argv += optind;
-
     if(argc<1){
+	fprintf(stderr,"No image file specified\n");
 	usage();
     }
 
-    if(num_certificates>0 && (opt_encrypt==0 && opt_add_public_key_to_passphrase==0)){
-	errx(1,"Encryption certificates specified by -e option not set. "
+    if(opt_just_print_encrypted_count && opt_just_print_unencrypted_count){
+	errx(1,"Options -j and -J conflict\n");
+    }
+
+    if(num_certificates>0 && (opt_encrypt==0 && opt_decrypt==0 && opt_add_public_key_to_passphrase==0)){
+	errx(1,"Encryption certificates specified but neither -e nor -d option not set. "
 	     "What do you want me to do with these certificates? ");
     }
 	    
-
     if((check_passphrase || passphrase_file) && opt_encrypt){
 	err(1,"Sorry, can't both encrypt and password crack. Pick one.\n");
     }
 
-    if(opt_encrypt && (new_passphrase==0 && num_certificates==0)){
-	err(1,"Currently -e requires that the passphrase be specified on the command line\nor that one or more encryption certificates be provided\n");
+    if(opt_encrypt && (new_passphrase==0 && num_certificates==0) && mode!=O_RDONLY){
+	err(1,"Currently -e requires that the passphrase be specified on the command line\n"
+	    "or that one or more encryption certificates be provided\n");
     }
 
     while(argc--){
@@ -296,7 +418,6 @@ int main(int argc,char **argv)
 	    else get_and_change_passphrase(fname);
 	}
 
-
 	/* Get the information */
 	AFFILE *af = af_open(fname,mode,0);
 	if(!af) af_err(1,"af_open(%s)",fname);
@@ -304,12 +425,11 @@ int main(int argc,char **argv)
 	    errx(1,"Cannot encrypt %s: %s only supports AFF and AFD files.",af_filename(af),progname);
 	}
 
-	af_vnode_info vni;
 	if(opt_encrypt && new_passphrase){
 	    int r = af_establish_aes_passphrase(af,new_passphrase);
 	    switch(r){
-	    case AF_ERROR_NO_AES: errx(1,"AFFLIB compiled without AES; cannot continue");
-	    case AF_ERROR_NO_SHA256: errx(1,"AFFLIB compiled without SHA256; cannot continue");
+	    case AF_ERROR_NO_AES: errx(1,"AFFLIB: AES256 not available; cannot continue");
+	    case AF_ERROR_NO_SHA256: errx(1,"AFFLIB: SHA256 not available; cannot continue");
 	    default: err(1,"%s: cannot establish passphrase (error %d)",fname,r);
 	    case 0: 
 	    case AF_ERROR_AFFKEY_EXISTS:
@@ -325,6 +445,14 @@ int main(int argc,char **argv)
 	    }
         }
 
+	if(opt_decrypt && old_passphrase){
+	    int r = af_use_aes_passphrase(af, old_passphrase);
+	    switch(r){
+	    case 0: printf("Passphrase is good!\n"); break;
+	    case AF_ERROR_WRONG_PASSPHRASE: errx(1,"%s: wrong passphrase",fname);
+	    }
+	}
+
 	if (opt_add_public_key_to_passphrase){ 
 	  if(!num_certificates) errx(1,"You must specify a certificate with the -C option");
 	  if(!check_passphrase) errx(1,"You must specify a passphrase with the -p option");
@@ -343,9 +471,25 @@ int main(int argc,char **argv)
 	    }
 	}
 	if(opt_encrypt){
-	    if(af_encrypt_unencrypted_unsigned_segments(af)){
+	    int count = 0;
+	    if(af_encrypt_unencrypted_nonsignature_segments(af,&count,mode)){
 		errx(1,"%s: can't encrypt unsigned, unencrypted segments",fname);
 	    }
+	    if(mode==O_RDONLY){		// if it is readonly just print the number of segments that would be changed.
+		printf("%d\n",count);
+		af_close(af);
+		continue;
+	    }
+	}
+	if(opt_decrypt){
+	    int count = 0;
+	    if(af_decrypt_encrypted_segments(af, &count, mode)){
+	    }
+	    if(mode==O_RDONLY){
+		printf("%d\n",count);
+		af_close(af);
+		continue;
+	    }
 	}
 
 	if(opt_add_passphrase_to_public_key) {
@@ -362,9 +506,23 @@ int main(int argc,char **argv)
 	}
 
 
+	af_vnode_info vni;
+	memset(&vni,0,sizeof(vni));
 	if(af_vstat(af,&vni)) err(1,"%s: af_vstat failed: ",fname);
 	const char *the_passphrase = 0;	// the correct passphrase
 
+	if(opt_just_print_encrypted_count){
+	    printf("%d\n",vni.segment_count_encrypted);
+	    af_close(af);
+	    continue;
+	}
+
+	if(opt_just_print_unencrypted_count){
+	    printf("%d\n",vni.segment_count_total-vni.segment_count_encrypted);
+	    af_close(af);
+	    continue;
+	}
+
 
 	/* were we supposed to try a check_passphrase? */
 	if(check_passphrase){
@@ -382,11 +540,29 @@ int main(int argc,char **argv)
 	    }
 	}
 	
-	printf("%s: %5d segments; %5d signed; %5d encrypted; ",
-	       fname,vni.segment_count_total,vni.segment_count_signed,vni.segment_count_encrypted);
-
-	if(the_passphrase) printf("passphrase correct (\"%s\")",the_passphrase);
-	putchar('\n');
+	if(opt_xml){
+	    /* This should be replaced with our xml.cpp object */
+	    printf("<afcrypto>\n");
+	    printf("  <image_filename>%s</image_filename>\n",fname);
+	    printf("  <segment_count_total>%d</segment_count_total>\n",vni.segment_count_total);
+	    printf("  <segment_count_signed>%d</segment_count_signed>\n",vni.segment_count_signed);
+	    printf("  <segment_count_encrypted>%d</segment_count_encrypted>\n",vni.segment_count_encrypted);
+	    printf("  <page_count_total>%d</page_count_total>\n",vni.page_count_total);
+	    printf("  <page_count_encrypted>%d</page_count_encrypted>\n",vni.page_count_encrypted);
+	    if(the_passphrase){
+		printf("  <passphrase correct='1'>%s</passphrase>\n",the_passphrase);
+	    }
+	    printf("</afcrypto>\n");
+	}
+	else{
+	    /* re-run vstat because counts may have changed */
+	    if(af_vstat(af,&vni)) err(1,"%s: af_vstat failed: ",fname);
+	    printf("%s: %5d segments; %5d signed; %5d encrypted; %5d pages; %5d encrypted pages",
+		   fname,vni.segment_count_total,vni.segment_count_signed,vni.segment_count_encrypted,
+		   vni.page_count_total,vni.page_count_encrypted );
+	    if(the_passphrase) printf("passphrase correct (\"%s\")",the_passphrase);
+	    putchar('\n');
+	}
 	af_close(af);
     }
     return(0);
