@@ -49,7 +49,6 @@
 #include "afflib.h"
 #include "afflib_i.h"
 #include "base64.h"
-#include "quads.h"
 
 #ifdef WIN32
 #include "unix4win32.h"
@@ -62,7 +61,6 @@
 #include <cstring>
 #endif
 
-
 using namespace std;
 
 #if HAVE_CTYPE_H
@@ -116,11 +114,16 @@ bool in_opt_j_list(char *segname)
     return false;
 }
 
+/* It's okay to print if there are no funky characters in it...
+ * (avoids problems with invalid UTF-8
+ */
 bool okay_to_print(const char *data,int datalen)
 {
     for(const char *cc=data;cc<data+datalen;cc++){
-	if (strchr("<>\r\n&'\"",*cc)) return false; // unprintable
-	if (!isprint(*cc)) return false;
+	if (*cc <= 0) return false;
+	if (*cc == 10 || *cc==13) continue;
+	if (*cc<32) return false;
+	if (*cc>=127) return false;
     }
     return true;
 }
@@ -151,13 +154,12 @@ int xml_info(const char *infile)
 
     printf("<!-- XML generated by afxml version %s -->\n",PACKAGE_VERSION);
     printf("<affinfo");
-    if(!opt_x) printf(" name='%s'",infile);
+    if(!opt_x) printf(" image_filename='%s'",infile);
     printf(">\n");
 
     af_rewind_seg(af);			// start at the beginning
 
     char segname[AF_MAX_NAME_LEN];
-    char segname2[AF_MAX_NAME_LEN];
     int  pages = 0;
     vector<string> seglist;		// list of segments we will get
     vector<int64_t> pagelist;		// list of segments we will get
@@ -192,7 +194,7 @@ int xml_info(const char *infile)
 	    size_t pagesize = af_page_size(af);
 	    size_t sectorsize = af_get_sectorsize(af);
 	    if(af_get_page(af,*it,data,&pagesize)){
-		err(1,"Can't read page "I64d,*it);
+		err(1,"Can't read page %"PRId64,*it);
 	    }
 	    psb.pages++;
 	    bool allblank = true;
@@ -236,7 +238,7 @@ int xml_info(const char *infile)
 	}
 
 	unsigned char *data= (unsigned char *)malloc(datalen);
-	if(data==0) err(1,"Can't allocate %d bytes for data",datalen);
+	if(data==0) err(1,"Can't allocate %zd bytes for data",datalen);
 	if(af_get_seg(af,segname,&arg,data,&datalen)!=0){
 	    err(1,"Can't read data for segment '%s'",segname);
 	}
@@ -246,7 +248,7 @@ int xml_info(const char *infile)
 	    if(!isalphanum(*cc)) *cc = '_';
 	}
 
-	if(datalen==8 && (arg & AF_SEG_QUADWORD || display_as_quad(segname))){
+	if(datalen==8 && (arg & AF_SEG_QUADWORD || af_display_as_quad(segname))){
 	    /* Print it as a 64-bit value.
 	     * The strcmp is there because early AF_IMAGESIZE segs didn't set
 	     * AF_SEG_QUADWORD...
@@ -287,7 +289,14 @@ int xml_info(const char *infile)
 	if(okay_to_print((const char *)data,datalen)){
 	    putchar('>');
 	    for(const char *cc=(const char *)data;cc<(const char *)data+datalen;cc++){
-		putchar(*cc);
+		switch(*cc){
+		case '>': fputs("&lt;",stdout);break;
+		case '<': fputs("&gt;",stdout);break;
+		case '&': fputs("&amp;",stdout);break;
+		case '\'': fputs("&apos;",stdout);break;
+		case '"': fputs("&quot;",stdout);break;
+		default: putchar(*cc);
+		}
 	    }
 	    printf("</%s>\n",segname);
 	    free(data);
@@ -298,6 +307,7 @@ int xml_info(const char *infile)
 	int  b64size = datalen*2+2;
 	char *b64buf = (char *)calloc(b64size,1);
 	int  b64size_real = b64_ntop(data,datalen,b64buf,b64size);
+	data[b64size_real] = 0;		// be sure it is null terminated
 
 	printf(" coding='base64'>");
 	fputs(b64buf,stdout);
